# -*- Python -*-

# Driver template for the LEMON parser generator.
# The author disclaims copyright to this source code.


# First off, define the token values.  These constants (all generated
# automatically by the parser generator) specify the various kinds of
# tokens (terminals) that the parser understands.
#
# Each symbol here is a terminal symbol in the grammar.

L_NAME                         =  1
T_COLON                        =  2
T_LSQUARE                      =  3
T_RSQUARE                      =  4
K_DEFINE                       =  5
K_ASSUME                       =  6
K_OBSERVE                      =  7
K_PREDICT                      =  8
K_CONFIGURE                    =  9
K_FORGET                       = 10
K_FREEZE                       = 11
K_REPORT                       = 12
K_INFER                        = 13
K_CLEAR                        = 14
K_ROLLBACK                     = 15
K_LIST_DIRECTIVES              = 16
K_GET_DIRECTIVE                = 17
K_FORCE                        = 18
K_SAMPLE                       = 19
K_CONTINUOUS_INFERENCE_STATUS  = 20
K_START_CONTINUOUS_INFERENCE   = 21
K_STOP_CONTINUOUS_INFERENCE    = 22
K_GET_CURRENT_EXCEPTION        = 23
K_GET_STATE                    = 24
K_GET_GLOBAL_LOGSCORE          = 25
K_PROFILER_CONFIGURE           = 26
K_PROFILER_CLEAR               = 27
K_PROFILER_LIST_RANDOM         = 28
K_CHOICES                      = 29
K_LOAD                         = 30
L_STRING                       = 31
L_INTEGER                      = 32
L_OPERATOR                     = 33
T_QUOTE                        = 34
T_BACKTICK                     = 35
T_COMMA                        = 36
T_LROUND                       = 37
T_RROUND                       = 38
T_TRUE                         = 39
T_FALSE                        = 40
L_REAL                         = 41
T_LANGLE                       = 42
T_RANGLE                       = 43
T_LCURLY                       = 44
T_RCURLY                       = 45

# The state of the parser is completely contained in an instance of
# the following class.

class Parser(object):

    # defaults
    YYERRORSYMBOL = None
    YYWILDCARD = None

    # The next thing included is series of definitions which control
    # various aspects of the generated parser.
    #    YYNOCODE           is a number which corresponds
    #                       to no legal terminal or nonterminal number.  This
    #                       number is used to fill in empty slots of the hash 
    #                       table.
    #    YYNSTATE           the combined number of states.
    #    YYNRULE            the number of rules in the grammar
    #    YYERRORSYMBOL      is the code number of the error symbol.  If not
    #                       defined, then do no error processing.

    YYNOCODE = 63
    YYNSTATE = 135
    YYNRULE = 76
    YYERRORSYMBOL = 46
    YY_NO_ACTION     = YYNSTATE + YYNRULE + 2
    YY_ACCEPT_ACTION = YYNSTATE + YYNRULE + 1
    YY_ERROR_ACTION  = YYNSTATE + YYNRULE


    # Next are that tables used to determine what action to take based on the
    # current state and lookahead token.  These tables are used to implement
    # functions that take a state number and lookahead value and return an
    # action integer.  
    #
    # Suppose the action integer is N.  Then the action is determined as
    # follows
    #
    #   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
    #                                      token onto the stack and goto state N.
    #
    #   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
    #
    #   N == YYNSTATE+YYNRULE              A syntax error has occurred.
    #
    #   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
    #
    #   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
    #                                      slots in the yy_action[] table.
    #
    # The action table is constructed as a single large table named yy_action[].
    # Given state S and lookahead X, the action is computed as
    #
    #      yy_action[ yy_shift_ofst[S] + X ]
    #
    # If the index value yy_shift_ofst[S]+X is out of range or if the value
    # yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
    # is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
    # and that yy_default[S] should be used instead.  
    #
    # The formula above is for computing the action when the lookahead is
    # a terminal symbol.  If the lookahead is a non-terminal (as occurs after
    # a reduce action) then the yy_reduce_ofst[] array is used in place of
    # the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
    # YY_SHIFT_USE_DFLT.
    #
    # The following are the tables generated in this section:
    #
    #  yy_action[]        A single table containing all actions.
    #  yy_lookahead[]     A table containing the lookahead for each entry in
    #                     yy_action.  Used to detect hash collisions.
    #  yy_shift_ofst[]    For each state, the offset into yy_action for
    #                     shifting terminals.
    #  yy_reduce_ofst[]   For each state, the offset into yy_action for
    #                     shifting non-terminals after a reduce.
    #  yy_default[]       Default action for each state.

    YY_ACTTAB_COUNT = 232
    yy_action = [
           59,   50,   18,   17,    7,   33,   32,   31,   16,   82, #     0
           81,   80,   30,   15,   14,   76,   13,   74,   73,   72, #    10
           71,    6,   69,   45,   62,   44,  136,   39,   25,    4, #    20
           43,  212,    2,   63,   37,   38,   66,  107,   59,   50, #    30
           18,   17,  109,   56,  133,  125,  124,   35,   46,   34, #    40
          113,   57,   48,   47,  125,  124,   86,   96,   98,  108, #    50
           22,   21,   20,   36,    3,  100,   99,   97,  135,   39, #    60
           43,    4,    3,  134,  132,  120,   66,  107,  125,  124, #    70
          130,  115,    9,  111,  125,  124,  114,   87,  110,  125, #    80
          124,  125,  124,   89,  122,   23,  125,  124,  117,   96, #    90
           98,  108,   22,   21,   20,   36,   58,  100,   99,   97, #   100
           70,  121,   51,  125,  124,    8,   26,   52,  101,  107, #   110
           27,  131,  107,   60,   40,   10,   11,   61,   41,  118, #   120
            9,  129,  116,   52,  106,  107,   96,   98,  108,   22, #   130
           21,   20,   36,  103,  100,   99,   97,   58,  112,  105, #   140
          107,   19,  104,  107,   95,  107,   29,  107,   93,  107, #   150
           83,  107,   28,  107,  102,   77,  107,   75,  107,   92, #   160
           91,   90,   68,   67,   42,   65,   24,   96,   98,  108, #   170
           22,   21,   20,   36,   49,  100,   99,   97,    1,  123, #   180
            1,   53,   64,    5,   55,   54,   88,  213,  213,   85, #   190
           84,   53,   79,  213,   94,  119,   78,  213,  213,  213, #   200
          213,  213,  213,  213,   96,   98,  128,  127,  128,  127, #   210
          213,  213,  100,   99,   97,  213,  126,  213,  126,   12, #   220
          213,   12, #   230
        ]
    yy_lookahead = [
            5,    6,    7,    8,    9,   10,   11,   12,   13,   14, #     0
           15,   16,   17,   18,   19,   20,   21,   22,   23,   24, #    10
           25,   26,   27,   28,    2,   30,    0,    1,    3,    3, #    20
           46,   47,   48,   49,   46,   46,   52,   53,    5,    6, #    30
            7,    8,   54,   46,    4,   57,   58,   59,   46,   60, #    40
           61,   54,   50,   51,   57,   58,    1,   31,   32,   33, #    50
           34,   35,   36,   37,   42,   39,   40,   41,    0,    1, #    60
           46,    3,   42,   49,    4,   54,   52,   53,   57,   58, #    70
           43,   54,    2,    4,   57,   58,   54,   32,   54,   57, #    80
           58,   57,   58,   54,    4,    1,   57,   58,   45,   31, #    90
           32,   33,   34,   35,   36,   37,    1,   39,   40,   41, #   100
           54,    4,   46,   57,   58,   36,   36,   46,   52,   53, #   110
           36,   52,   53,   46,   46,    2,   36,   50,   50,   45, #   120
            2,   43,   61,   46,   52,   53,   31,   32,   33,   34, #   130
           35,   36,   37,   38,   39,   40,   41,    1,   61,   52, #   140
           53,    1,   52,   53,   52,   53,   52,   53,   52,   53, #   150
           52,   53,   52,   53,   38,   52,   53,   52,   53,    4, #   160
            4,    4,   29,   31,    2,    4,    3,   31,   32,   33, #   170
           34,   35,   36,   37,    1,   39,   40,   41,    3,    4, #   180
            3,   31,    4,   56,   46,   46,   55,   62,   62,   55, #   190
           55,   31,   55,   62,   53,   45,   53,   62,   62,   62, #   200
           62,   62,   62,   62,   31,   32,   31,   32,   31,   32, #   210
           62,   62,   39,   40,   41,   62,   41,   62,   41,   44, #   220
           62,   44, #   230
        ]
    YY_SHIFT_USE_DFLT = -6
    YY_SHIFT_COUNT = 62
    YY_SHIFT_MIN = -5
    YY_SHIFT_MAX = 188
    yy_shift_ofst = [
           68,  185,   26,  187,   -5,  105,  187,  187,  187,  187, #     0
          187,  187,  160,  146,  146,  146,  146,  146,  146,  146, #    10
          146,  146,  146,  146,   33,   33,  170,  170,  183,  183, #    20
           55,   55,   55,   55,   84,   90,   -6,   79,   80,   22, #    30
          188,  171,  173,  172,  142,  143,  167,  166,  165,   30, #    40
          150,  126,  128,  123,   53,  107,   88,   37,   30,   94, #    50
           70,   40,   25, #    60
        ]
    YY_REDUCE_USE_DFLT = -17
    YY_REDUCE_COUNT = 36
    YY_REDUCE_MIN = -16
    YY_REDUCE_MAX = 153
    yy_reduce_ofst = [
          -16,  -12,   24,   -3,    2,   66,   56,   39,   34,   32, #     0
           27,   21,  -11,  115,  113,  110,  108,  106,  104,  102, #    10
          100,   97,   82,   69,   78,   77,   87,   71,  153,  151, #    20
          147,  145,  144,  141,  149,  148,  137, #    30
        ]
    yy_default = [
          211,  211,  211,  211,  211,  211,  211,  211,  211,  211, #     0
          211,  211,  211,  211,  211,  211,  211,  211,  211,  211, #    10
          211,  211,  211,  211,  211,  211,  211,  211,  211,  211, #    20
          211,  211,  211,  211,  211,  211,  182,  211,  211,  174, #    30
          211,  211,  211,  211,  211,  211,  211,  211,  211,  211, #    40
          211,  211,  211,  211,  211,  211,  211,  211,  174,  211, #    50
          211,  211,  211,  137,  145,  143,  142,  171,  170,  169, #    60
          168,  167,  166,  165,  164,  163,  162,  161,  160,  159, #    70
          158,  157,  156,  155,  154,  153,  173,  172,  152,  151, #    80
          146,  141,  140,  150,  149,  148,  188,  187,  186,  185, #    90
          184,  183,  181,  180,  179,  178,  177,  176,  175,  200, #   100
          202,  199,  208,  206,  210,  209,  207,  205,  204,  203, #   110
          201,  198,  197,  196,  195,  194,  193,  192,  191,  190, #   120
          189,  147,  144,  139,  138, #   130
        ]


    # The next table maps tokens into fallback tokens.  If a construct
    # like the following:
    #
    #      %fallback ID X Y Z.
    #
    # appears in the grammer, then ID becomes a fallback token for X, Y,
    # and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
    # but it does not parse, the type of the token is changed to ID and
    # the parse is retried before an error is thrown.

    yyFallback = [
          0,  #          $ => nothing
          0,  #     L_NAME => nothing
          0,  #    T_COLON => nothing
          0,  #  T_LSQUARE => nothing
          0,  #  T_RSQUARE => nothing
          0,  #   K_DEFINE => nothing
          1,  #   K_ASSUME => L_NAME
          1,  #  K_OBSERVE => L_NAME
          1,  #  K_PREDICT => L_NAME
          1,  # K_CONFIGURE => L_NAME
          1,  #   K_FORGET => L_NAME
          1,  #   K_FREEZE => L_NAME
          1,  #   K_REPORT => L_NAME
          1,  #    K_INFER => L_NAME
          1,  #    K_CLEAR => L_NAME
          1,  # K_ROLLBACK => L_NAME
          1,  # K_LIST_DIRECTIVES => L_NAME
          1,  # K_GET_DIRECTIVE => L_NAME
          1,  #    K_FORCE => L_NAME
          1,  #   K_SAMPLE => L_NAME
          1,  # K_CONTINUOUS_INFERENCE_STATUS => L_NAME
          1,  # K_START_CONTINUOUS_INFERENCE => L_NAME
          1,  # K_STOP_CONTINUOUS_INFERENCE => L_NAME
          1,  # K_GET_CURRENT_EXCEPTION => L_NAME
          1,  # K_GET_STATE => L_NAME
          1,  # K_GET_GLOBAL_LOGSCORE => L_NAME
          1,  # K_PROFILER_CONFIGURE => L_NAME
          1,  # K_PROFILER_CLEAR => L_NAME
          1,  # K_PROFILER_LIST_RANDOM => L_NAME
          1,  #  K_CHOICES => L_NAME
          1,  #     K_LOAD => L_NAME
          0,  #   L_STRING => nothing
          0,  #  L_INTEGER => nothing
          0,  # L_OPERATOR => nothing
          0,  #    T_QUOTE => nothing
          0,  # T_BACKTICK => nothing
          0,  #    T_COMMA => nothing
          0,  #   T_LROUND => nothing
          0,  #   T_RROUND => nothing
          0,  #     T_TRUE => nothing
          0,  #    T_FALSE => nothing
          0,  #     L_REAL => nothing
         33,  #   T_LANGLE => L_OPERATOR
         33,  #   T_RANGLE => L_OPERATOR
        ]


    # The following structure represents a single element of the
    # parser's stack.  Information stored includes:
    #
    #   +  The state number for the parser at this level of the stack.
    #
    #   +  The value of the token stored at this level of the stack.
    #      (In other words, the "major" token.)
    #
    #   +  The semantic value stored at this level of the stack.  This is
    #      the information used by the action routines in the grammar.
    #      It is sometimes called the "minor" token.
    #
    class yyStackEntry(object):
        def __init__(
            self,
            stateno, # The state-number
            major,   # The major token value.  This is the code
                     # number for the token at this stack level
            minor,   # The user-supplied minor token value.  This
                     # is the value of the token
            ):
            self.stateno = stateno
            self.major = major
            self.minor = minor
            return


    yyTraceFILE = None
    yyTracePrompt = None

    def trace(self, TraceFILE, zTracePrompt):
        '''Turn parser tracing on by giving a stream to which to write
        the trace and a prompt to preface each trace message.  Tracing
        is turned off by making either argument None.
        '''
        self.yyTraceFILE = TraceFILE
        self.yyTracePrompt = zTracePrompt
        if self.yyTraceFILE is None:
            self.yyTracePrompt = None
        elif self.yyTracePrompt is None:
            self.yyTraceFILE = None
        return


    # For tracing shifts, the names of all terminals and nonterminals
    # are required.  The following table supplies these names
    yyTokenName = [
        "$",                   "L_NAME",              "T_COLON",             "T_LSQUARE",   
        "T_RSQUARE",           "K_DEFINE",            "K_ASSUME",            "K_OBSERVE",   
        "K_PREDICT",           "K_CONFIGURE",         "K_FORGET",            "K_FREEZE",    
        "K_REPORT",            "K_INFER",             "K_CLEAR",             "K_ROLLBACK",  
        "K_LIST_DIRECTIVES",        "K_GET_DIRECTIVE",        "K_FORCE",             "K_SAMPLE",    
        "K_CONTINUOUS_INFERENCE_STATUS",        "K_START_CONTINUOUS_INFERENCE",        "K_STOP_CONTINUOUS_INFERENCE",        "K_GET_CURRENT_EXCEPTION",
        "K_GET_STATE",         "K_GET_GLOBAL_LOGSCORE",        "K_PROFILER_CONFIGURE",        "K_PROFILER_CLEAR",
        "K_PROFILER_LIST_RANDOM",        "K_CHOICES",           "K_LOAD",              "L_STRING",    
        "L_INTEGER",           "L_OPERATOR",          "T_QUOTE",             "T_BACKTICK",  
        "T_COMMA",             "T_LROUND",            "T_RROUND",            "T_TRUE",      
        "T_FALSE",             "L_REAL",              "T_LANGLE",            "T_RANGLE",    
        "T_LCURLY",            "T_RCURLY",            "error",               "venture",     
        "instructions",        "instruction",         "directive",           "command",     
        "expression",          "literal",             "json",                "directive_ref",
        "expressions",         "json_list",           "json_dict",           "json_list_terms",
        "json_dict_entries",        "json_dict_entry",
        ]

    # For tracing reduce actions, the names of all rules are required.
    yyRuleName = [
        "venture ::=", #   0
        "venture ::= instructions", #   1
        "instructions ::= instruction", #   2
        "instructions ::= instructions instruction", #   3
        "instruction ::= L_NAME T_COLON T_LSQUARE directive T_RSQUARE", #   4
        "instruction ::= T_LSQUARE directive T_RSQUARE", #   5
        "instruction ::= T_LSQUARE command T_RSQUARE", #   6
        "instruction ::= expression", #   7
        "instruction ::= error T_COLON T_LSQUARE directive T_RSQUARE", #   8
        "instruction ::= L_NAME T_COLON T_LSQUARE error T_RSQUARE", #   9
        "instruction ::= error T_COLON T_LSQUARE error T_RSQUARE", #  10
        "instruction ::= T_LSQUARE error T_RSQUARE", #  11
        "directive ::= K_DEFINE L_NAME expression", #  12
        "directive ::= K_ASSUME L_NAME expression", #  13
        "directive ::= K_OBSERVE expression literal", #  14
        "directive ::= K_PREDICT expression", #  15
        "command ::= K_CONFIGURE json", #  16
        "command ::= K_FORGET directive_ref", #  17
        "command ::= K_FREEZE directive_ref", #  18
        "command ::= K_REPORT directive_ref", #  19
        "command ::= K_INFER expression", #  20
        "command ::= K_CLEAR", #  21
        "command ::= K_ROLLBACK", #  22
        "command ::= K_LIST_DIRECTIVES", #  23
        "command ::= K_GET_DIRECTIVE directive_ref", #  24
        "command ::= K_FORCE expression literal", #  25
        "command ::= K_SAMPLE expression", #  26
        "command ::= K_CONTINUOUS_INFERENCE_STATUS", #  27
        "command ::= K_START_CONTINUOUS_INFERENCE expression", #  28
        "command ::= K_STOP_CONTINUOUS_INFERENCE", #  29
        "command ::= K_GET_CURRENT_EXCEPTION", #  30
        "command ::= K_GET_STATE", #  31
        "command ::= K_GET_GLOBAL_LOGSCORE", #  32
        "command ::= K_PROFILER_CONFIGURE json", #  33
        "command ::= K_PROFILER_CLEAR", #  34
        "command ::= K_PROFILER_LIST_RANDOM K_CHOICES", #  35
        "command ::= K_LOAD L_STRING", #  36
        "directive_ref ::= L_INTEGER", #  37
        "directive_ref ::= L_NAME", #  38
        "expression ::= L_NAME", #  39
        "expression ::= L_OPERATOR", #  40
        "expression ::= literal", #  41
        "expression ::= T_QUOTE expression", #  42
        "expression ::= T_BACKTICK expression", #  43
        "expression ::= T_COMMA expression", #  44
        "expression ::= T_LROUND expressions T_RROUND", #  45
        "expression ::= T_LROUND expressions error T_RROUND", #  46
        "expressions ::=", #  47
        "expressions ::= expressions expression", #  48
        "literal ::= T_TRUE", #  49
        "literal ::= T_FALSE", #  50
        "literal ::= L_INTEGER", #  51
        "literal ::= L_REAL", #  52
        "literal ::= L_STRING", #  53
        "literal ::= L_NAME T_LANGLE json T_RANGLE", #  54
        "literal ::= L_NAME T_LANGLE error T_RANGLE", #  55
        "json ::= L_STRING", #  56
        "json ::= L_INTEGER", #  57
        "json ::= L_REAL", #  58
        "json ::= json_list", #  59
        "json ::= json_dict", #  60
        "json_list ::= T_LSQUARE T_RSQUARE", #  61
        "json_list ::= T_LSQUARE json_list_terms T_RSQUARE", #  62
        "json_list ::= T_LSQUARE json_list_terms error T_RSQUARE", #  63
        "json_list ::= T_LSQUARE error T_RSQUARE", #  64
        "json_list_terms ::= json", #  65
        "json_list_terms ::= json_list_terms T_COMMA json", #  66
        "json_list_terms ::= error T_COMMA json", #  67
        "json_dict ::= T_LCURLY T_RCURLY", #  68
        "json_dict ::= T_LCURLY json_dict_entries T_RCURLY", #  69
        "json_dict ::= T_LCURLY json_dict_entries error T_RCURLY", #  70
        "json_dict_entries ::= json_dict_entry", #  71
        "json_dict_entries ::= json_dict_entries T_COMMA json_dict_entry", #  72
        "json_dict_entries ::= error T_COMMA json_dict_entry", #  73
        "json_dict_entry ::= L_STRING T_COLON json", #  74
        "json_dict_entry ::= error T_COLON json", #  75
        ]


    def __init__(self, delegate):
        self.yystack = [] # The parser's stack
        self.delegate = delegate
        return


    def yy_pop_parser_stack(self):
        """Pop the parser's stack once. Return the major token number
        for the symbol popped.
        """
        if not self.yystack:
            return 0
        yytos = self.yystack.pop()
        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sPopping %s\n" % (
                self.yyTracePrompt,
                self.yyTokenName[yytos.major]))
        yymajor = yytos.major
        return yymajor


    def yy_find_shift_action(self,       # The parser
                             iLookAhead  # The look-ahead token
                             ):
        '''Find the appropriate action for a parser given the terminal
        look-ahead token iLookAhead.

        If the look-ahead token is YYNOCODE, then check to see if the
        action is independent of the look-ahead.  If it is, return the
        action, otherwise return YY_NO_ACTION.
        '''
        yyTraceFILE = self.yyTraceFILE
        stateno = self.yystack[-1].stateno
        if stateno > self.YY_SHIFT_COUNT:
            return self.yy_default[stateno]
        i = self.yy_shift_ofst[stateno]
        if i == self.YY_SHIFT_USE_DFLT:
            return self.yy_default[stateno]
        assert iLookAhead != self.YYNOCODE
        i += iLookAhead
        if i < 0 or i >= self.YY_ACTTAB_COUNT or self.yy_lookahead[i] != iLookAhead:
            if iLookAhead > 0:
                yyFallback = self.yyFallback
                yyTokenName = self.yyTokenName
                if iLookAhead < len(yyFallback):
                    iFallback = yyFallback[iLookAhead] # Fallback token
                    if iFallback != 0:
                        if yyTraceFILE:
                            yyTraceFILE.write(
                                "%sFALLBACK %s => %s\n" %
                                (self.yyTracePrompt,
                                 yyTokenName[iLookAhead], yyTokenName[iFallback]))
                        return self.yy_find_shift_action(iFallback);
                YYWILDCARD = self.YYWILDCARD
                if YYWILDCARD is not None:
                    j = i - iLookAhead + YYWILDCARD
                    # [TRC 20150122: Lemon avoids generating tests
                    # against j if they can never fail, based on
                    # compile-time values here, presumably to limit
                    # compiler warnings.  Not so easy to do in Python
                    # so we'll just skip that.]
                    if j >= 0 and j < self.YY_ACTTAB_COUNT and self.yy_lookahead[j] == YYWILDCARD:
                        if yyTraceFILE:
                            yyTraceFILE.write(
                                "%sWILDCARD %s => %s\n" %
                                (self.yyTracePrompt,
                                 yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]))
                        return self.yy_action[j];
            return self.yy_default[stateno]
        else:
            return self.yy_action[i]


    def yy_find_reduce_action(self,
                              stateno,    # Current state number
                              iLookAhead  # The look-ahead token
                              ):
        '''Find the appropriate action for a parser given the
        non-terminal look-ahead token iLookAhead.
        
        If the look-ahead token is YYNOCODE, then check to see if the
        action is independent of the look-ahead.  If it is, return the
        action, otherwise return YY_NO_ACTION.
        '''
        YYERRORSYMBOL = self.YYERRORSYMBOL
        if YYERRORSYMBOL is not None:
            if stateno > self.YY_REDUCE_COUNT:
                return self.yy_default[stateno]
        else:
            assert stateno <= self.YY_REDUCE_COUNT
        i = self.yy_reduce_ofst[stateno]
        assert i != self.YY_REDUCE_USE_DFLT
        assert iLookAhead != self.YYNOCODE
        i += iLookAhead
        if YYERRORSYMBOL is not None:
            if i < 0 or i >= self.YY_ACTTAB_COUNT or self.yy_lookahead[i] != iLookAhead:
                return self.yy_default[stateno]
        else:
            assert i >= 0 and i < self.YY_ACTTAB_COUNT
            assert self.yy_lookahead[i] == iLookAhead
        return self.yy_action[i]


    def yy_shift(self,        # The parser to be shifted
                 yyNewState,  # The new state to shift in
                 yyMajor,     # The major token to shift in
                 yyMinor      # The minor token to shift in
                 ):
        '''Perform a shift action.'''

        yytos = self.yyStackEntry(
            stateno = yyNewState,
            major = yyMajor,
            minor = yyMinor
            )
        self.yystack.append(yytos)

        yyTraceFILE = self.yyTraceFILE
        if yyTraceFILE:
            yyTraceFILE.write("%sShift %d\n" % (self.yyTracePrompt, yyNewState))
            yyTraceFILE.write("%sStack:" % self.yyTracePrompt)
            for entry in self.yystack:
                yyTraceFILE.write(" %s" % self.yyTokenName[entry.major])
            yyTraceFILE.write("\n")

        return


    # The following table contains information about every rule that
    # is used during the reduce.
    from collections import namedtuple
    yyRuleInfoEntry = namedtuple(
        'yyRuleInfoEntry',
        ('lhs',  # Symbol on the left-hand side of the rule
         'nrhs', # Number of right-hand side symbols in the rule
         ))
    yyRuleInfo = [
        yyRuleInfoEntry( 47, 0 ),
        yyRuleInfoEntry( 47, 1 ),
        yyRuleInfoEntry( 48, 1 ),
        yyRuleInfoEntry( 48, 2 ),
        yyRuleInfoEntry( 49, 5 ),
        yyRuleInfoEntry( 49, 3 ),
        yyRuleInfoEntry( 49, 3 ),
        yyRuleInfoEntry( 49, 1 ),
        yyRuleInfoEntry( 49, 5 ),
        yyRuleInfoEntry( 49, 5 ),
        yyRuleInfoEntry( 49, 5 ),
        yyRuleInfoEntry( 49, 3 ),
        yyRuleInfoEntry( 50, 3 ),
        yyRuleInfoEntry( 50, 3 ),
        yyRuleInfoEntry( 50, 3 ),
        yyRuleInfoEntry( 50, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 3 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 1 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 51, 2 ),
        yyRuleInfoEntry( 55, 1 ),
        yyRuleInfoEntry( 55, 1 ),
        yyRuleInfoEntry( 52, 1 ),
        yyRuleInfoEntry( 52, 1 ),
        yyRuleInfoEntry( 52, 1 ),
        yyRuleInfoEntry( 52, 2 ),
        yyRuleInfoEntry( 52, 2 ),
        yyRuleInfoEntry( 52, 2 ),
        yyRuleInfoEntry( 52, 3 ),
        yyRuleInfoEntry( 52, 4 ),
        yyRuleInfoEntry( 56, 0 ),
        yyRuleInfoEntry( 56, 2 ),
        yyRuleInfoEntry( 53, 1 ),
        yyRuleInfoEntry( 53, 1 ),
        yyRuleInfoEntry( 53, 1 ),
        yyRuleInfoEntry( 53, 1 ),
        yyRuleInfoEntry( 53, 1 ),
        yyRuleInfoEntry( 53, 4 ),
        yyRuleInfoEntry( 53, 4 ),
        yyRuleInfoEntry( 54, 1 ),
        yyRuleInfoEntry( 54, 1 ),
        yyRuleInfoEntry( 54, 1 ),
        yyRuleInfoEntry( 54, 1 ),
        yyRuleInfoEntry( 54, 1 ),
        yyRuleInfoEntry( 57, 2 ),
        yyRuleInfoEntry( 57, 3 ),
        yyRuleInfoEntry( 57, 4 ),
        yyRuleInfoEntry( 57, 3 ),
        yyRuleInfoEntry( 59, 1 ),
        yyRuleInfoEntry( 59, 3 ),
        yyRuleInfoEntry( 59, 3 ),
        yyRuleInfoEntry( 58, 2 ),
        yyRuleInfoEntry( 58, 3 ),
        yyRuleInfoEntry( 58, 4 ),
        yyRuleInfoEntry( 60, 1 ),
        yyRuleInfoEntry( 60, 3 ),
        yyRuleInfoEntry( 60, 3 ),
        yyRuleInfoEntry( 61, 3 ),
        yyRuleInfoEntry( 61, 3 ),
        ]


    # Action code for each rule follows.
    def action_000(self):
        # venture ::=
        return self.delegate.p_venture_empty(
            )
    def action_001(self):
        # venture ::= instructions
        return self.delegate.p_venture_i(
            insts = self.yystack[-1].minor,
            )
    def action_002(self):
        # instructions ::= instruction
        return self.delegate.p_instructions_one(
            inst = self.yystack[-1].minor,
            )
    def action_003(self):
        # instructions ::= instructions instruction
        return self.delegate.p_instructions_many(
            insts = self.yystack[-2].minor,
            inst = self.yystack[-1].minor,
            )
    def action_004(self):
        # instruction ::= L_NAME T_COLON T_LSQUARE directive T_RSQUARE
        return self.delegate.p_instruction_labelled(
            l = self.yystack[-5].minor,
            open = self.yystack[-3].minor,
            d = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_005(self):
        # instruction ::= T_LSQUARE directive T_RSQUARE
        return self.delegate.p_instruction_unlabelled(
            open = self.yystack[-3].minor,
            d = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_006(self):
        # instruction ::= T_LSQUARE command T_RSQUARE
        return self.delegate.p_instruction_command(
            open = self.yystack[-3].minor,
            c = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_007(self):
        # instruction ::= expression
        return self.delegate.p_instruction_expression(
            e = self.yystack[-1].minor,
            )
    def action_008(self):
        # instruction ::= error T_COLON T_LSQUARE directive T_RSQUARE
        return self.delegate.p_instruction_laberror(
            colon = self.yystack[-4].minor,
            open = self.yystack[-3].minor,
            d = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_009(self):
        # instruction ::= L_NAME T_COLON T_LSQUARE error T_RSQUARE
        return self.delegate.p_instruction_direrror(
            l = self.yystack[-5].minor,
            colon = self.yystack[-4].minor,
            open = self.yystack[-3].minor,
            close = self.yystack[-1].minor,
            )
    def action_010(self):
        # instruction ::= error T_COLON T_LSQUARE error T_RSQUARE
        return self.delegate.p_instruction_labdirerror(
            colon = self.yystack[-4].minor,
            open = self.yystack[-3].minor,
            close = self.yystack[-1].minor,
            )
    def action_011(self):
        # instruction ::= T_LSQUARE error T_RSQUARE
        return self.delegate.p_instruction_error(
            open = self.yystack[-3].minor,
            close = self.yystack[-1].minor,
            )
    def action_012(self):
        # directive ::= K_DEFINE L_NAME expression
        return self.delegate.p_directive_define(
            k = self.yystack[-3].minor,
            n = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_013(self):
        # directive ::= K_ASSUME L_NAME expression
        return self.delegate.p_directive_assume(
            k = self.yystack[-3].minor,
            n = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_014(self):
        # directive ::= K_OBSERVE expression literal
        return self.delegate.p_directive_observe(
            k = self.yystack[-3].minor,
            e = self.yystack[-2].minor,
            v = self.yystack[-1].minor,
            )
    def action_015(self):
        # directive ::= K_PREDICT expression
        return self.delegate.p_directive_predict(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_016(self):
        # command ::= K_CONFIGURE json
        return self.delegate.p_command_configure(
            k = self.yystack[-2].minor,
            options = self.yystack[-1].minor,
            )
    def action_017(self):
        # command ::= K_FORGET directive_ref
        return self.delegate.p_command_forget(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_018(self):
        # command ::= K_FREEZE directive_ref
        return self.delegate.p_command_freeze(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_019(self):
        # command ::= K_REPORT directive_ref
        return self.delegate.p_command_report(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_020(self):
        # command ::= K_INFER expression
        return self.delegate.p_command_infer(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_021(self):
        # command ::= K_CLEAR
        return self.delegate.p_command_clear(
            k = self.yystack[-1].minor,
            )
    def action_022(self):
        # command ::= K_ROLLBACK
        return self.delegate.p_command_rollback(
            k = self.yystack[-1].minor,
            )
    def action_023(self):
        # command ::= K_LIST_DIRECTIVES
        return self.delegate.p_command_list_directives(
            k = self.yystack[-1].minor,
            )
    def action_024(self):
        # command ::= K_GET_DIRECTIVE directive_ref
        return self.delegate.p_command_get_directive(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_025(self):
        # command ::= K_FORCE expression literal
        return self.delegate.p_command_force(
            k = self.yystack[-3].minor,
            e = self.yystack[-2].minor,
            v = self.yystack[-1].minor,
            )
    def action_026(self):
        # command ::= K_SAMPLE expression
        return self.delegate.p_command_sample(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_027(self):
        # command ::= K_CONTINUOUS_INFERENCE_STATUS
        return self.delegate.p_command_continuous_inference_status(
            k = self.yystack[-1].minor,
            )
    def action_028(self):
        # command ::= K_START_CONTINUOUS_INFERENCE expression
        return self.delegate.p_command_start_continuous_inference(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_029(self):
        # command ::= K_STOP_CONTINUOUS_INFERENCE
        return self.delegate.p_command_stop_continuous_inference(
            k = self.yystack[-1].minor,
            )
    def action_030(self):
        # command ::= K_GET_CURRENT_EXCEPTION
        return self.delegate.p_command_get_current_exception(
            k = self.yystack[-1].minor,
            )
    def action_031(self):
        # command ::= K_GET_STATE
        return self.delegate.p_command_get_state(
            k = self.yystack[-1].minor,
            )
    def action_032(self):
        # command ::= K_GET_GLOBAL_LOGSCORE
        return self.delegate.p_command_get_global_logscore(
            k = self.yystack[-1].minor,
            )
    def action_033(self):
        # command ::= K_PROFILER_CONFIGURE json
        return self.delegate.p_command_profiler_configure(
            k = self.yystack[-2].minor,
            options = self.yystack[-1].minor,
            )
    def action_034(self):
        # command ::= K_PROFILER_CLEAR
        return self.delegate.p_command_profiler_clear(
            k = self.yystack[-1].minor,
            )
    def action_035(self):
        # command ::= K_PROFILER_LIST_RANDOM K_CHOICES
        return self.delegate.p_command_profiler_list_random(
            k = self.yystack[-2].minor,
            )
    def action_036(self):
        # command ::= K_LOAD L_STRING
        return self.delegate.p_command_load(
            k = self.yystack[-2].minor,
            pathname = self.yystack[-1].minor,
            )
    def action_037(self):
        # directive_ref ::= L_INTEGER
        return self.delegate.p_directive_ref_numbered(
            number = self.yystack[-1].minor,
            )
    def action_038(self):
        # directive_ref ::= L_NAME
        return self.delegate.p_directive_ref_labelled(
            label = self.yystack[-1].minor,
            )
    def action_039(self):
        # expression ::= L_NAME
        return self.delegate.p_expression_symbol(
            name = self.yystack[-1].minor,
            )
    def action_040(self):
        # expression ::= L_OPERATOR
        return self.delegate.p_expression_operator(
            op = self.yystack[-1].minor,
            )
    def action_041(self):
        # expression ::= literal
        return self.delegate.p_expression_literal(
            value = self.yystack[-1].minor,
            )
    def action_042(self):
        # expression ::= T_QUOTE expression
        return self.delegate.p_expression_quote(
            quote = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_043(self):
        # expression ::= T_BACKTICK expression
        return self.delegate.p_expression_qquote(
            qquote = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_044(self):
        # expression ::= T_COMMA expression
        return self.delegate.p_expression_unquote(
            unquote = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_045(self):
        # expression ::= T_LROUND expressions T_RROUND
        return self.delegate.p_expression_combination(
            open = self.yystack[-3].minor,
            es = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_046(self):
        # expression ::= T_LROUND expressions error T_RROUND
        return self.delegate.p_expression_comb_error(
            open = self.yystack[-4].minor,
            es = self.yystack[-3].minor,
            close = self.yystack[-1].minor,
            )
    def action_047(self):
        # expressions ::=
        return self.delegate.p_expressions_none(
            )
    def action_048(self):
        # expressions ::= expressions expression
        return self.delegate.p_expressions_some(
            es = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_049(self):
        # literal ::= T_TRUE
        return self.delegate.p_literal_true(
            t = self.yystack[-1].minor,
            )
    def action_050(self):
        # literal ::= T_FALSE
        return self.delegate.p_literal_false(
            f = self.yystack[-1].minor,
            )
    def action_051(self):
        # literal ::= L_INTEGER
        return self.delegate.p_literal_integer(
            v = self.yystack[-1].minor,
            )
    def action_052(self):
        # literal ::= L_REAL
        return self.delegate.p_literal_real(
            v = self.yystack[-1].minor,
            )
    def action_053(self):
        # literal ::= L_STRING
        return self.delegate.p_literal_string(
            v = self.yystack[-1].minor,
            )
    def action_054(self):
        # literal ::= L_NAME T_LANGLE json T_RANGLE
        return self.delegate.p_literal_json(
            type = self.yystack[-4].minor,
            open = self.yystack[-3].minor,
            value = self.yystack[-2].minor,
            close = self.yystack[-1].minor,
            )
    def action_055(self):
        # literal ::= L_NAME T_LANGLE error T_RANGLE
        return self.delegate.p_literal_json_error(
            type = self.yystack[-4].minor,
            open = self.yystack[-3].minor,
            close = self.yystack[-1].minor,
            )
    def action_056(self):
        # json ::= L_STRING
        return self.delegate.p_json_string(
            v = self.yystack[-1].minor,
            )
    def action_057(self):
        # json ::= L_INTEGER
        return self.delegate.p_json_integer(
            v = self.yystack[-1].minor,
            )
    def action_058(self):
        # json ::= L_REAL
        return self.delegate.p_json_real(
            v = self.yystack[-1].minor,
            )
    def action_059(self):
        # json ::= json_list
        return self.delegate.p_json_list(
            l = self.yystack[-1].minor,
            )
    def action_060(self):
        # json ::= json_dict
        return self.delegate.p_json_dict(
            d = self.yystack[-1].minor,
            )
    def action_061(self):
        # json_list ::= T_LSQUARE T_RSQUARE
        return self.delegate.p_json_list_empty(
            )
    def action_062(self):
        # json_list ::= T_LSQUARE json_list_terms T_RSQUARE
        return self.delegate.p_json_list_nonempty(
            ts = self.yystack[-2].minor,
            )
    def action_063(self):
        # json_list ::= T_LSQUARE json_list_terms error T_RSQUARE
        return self.delegate.p_json_list_error1(
            ts = self.yystack[-3].minor,
            )
    def action_064(self):
        # json_list ::= T_LSQUARE error T_RSQUARE
        return self.delegate.p_json_list_error(
            )
    def action_065(self):
        # json_list_terms ::= json
        return self.delegate.p_json_list_terms_one(
            t = self.yystack[-1].minor,
            )
    def action_066(self):
        # json_list_terms ::= json_list_terms T_COMMA json
        return self.delegate.p_json_list_terms_many(
            ts = self.yystack[-3].minor,
            t = self.yystack[-1].minor,
            )
    def action_067(self):
        # json_list_terms ::= error T_COMMA json
        return self.delegate.p_json_list_terms_error(
            t = self.yystack[-1].minor,
            )
    def action_068(self):
        # json_dict ::= T_LCURLY T_RCURLY
        return self.delegate.p_json_dict_empty(
            )
    def action_069(self):
        # json_dict ::= T_LCURLY json_dict_entries T_RCURLY
        return self.delegate.p_json_dict_nonempty(
            es = self.yystack[-2].minor,
            )
    def action_070(self):
        # json_dict ::= T_LCURLY json_dict_entries error T_RCURLY
        return self.delegate.p_json_dict_error(
            es = self.yystack[-3].minor,
            )
    def action_071(self):
        # json_dict_entries ::= json_dict_entry
        return self.delegate.p_json_dict_entries_one(
            e = self.yystack[-1].minor,
            )
    def action_072(self):
        # json_dict_entries ::= json_dict_entries T_COMMA json_dict_entry
        return self.delegate.p_json_dict_entries_many(
            es = self.yystack[-3].minor,
            e = self.yystack[-1].minor,
            )
    def action_073(self):
        # json_dict_entries ::= error T_COMMA json_dict_entry
        return self.delegate.p_json_dict_entries_error(
            e = self.yystack[-1].minor,
            )
    def action_074(self):
        # json_dict_entry ::= L_STRING T_COLON json
        return self.delegate.p_json_dict_entry_e(
            key = self.yystack[-3].minor,
            value = self.yystack[-1].minor,
            )
    def action_075(self):
        # json_dict_entry ::= error T_COLON json
        return self.delegate.p_json_dict_entry_error(
            value = self.yystack[-1].minor,
            )
    yy_action_method = [
        action_000,
        action_001,
        action_002,
        action_003,
        action_004,
        action_005,
        action_006,
        action_007,
        action_008,
        action_009,
        action_010,
        action_011,
        action_012,
        action_013,
        action_014,
        action_015,
        action_016,
        action_017,
        action_018,
        action_019,
        action_020,
        action_021,
        action_022,
        action_023,
        action_024,
        action_025,
        action_026,
        action_027,
        action_028,
        action_029,
        action_030,
        action_031,
        action_032,
        action_033,
        action_034,
        action_035,
        action_036,
        action_037,
        action_038,
        action_039,
        action_040,
        action_041,
        action_042,
        action_043,
        action_044,
        action_045,
        action_046,
        action_047,
        action_048,
        action_049,
        action_050,
        action_051,
        action_052,
        action_053,
        action_054,
        action_055,
        action_056,
        action_057,
        action_058,
        action_059,
        action_060,
        action_061,
        action_062,
        action_063,
        action_064,
        action_065,
        action_066,
        action_067,
        action_068,
        action_069,
        action_070,
        action_071,
        action_072,
        action_073,
        action_074,
        action_075,
    ]


    def yy_reduce(self,     # The parser
                  yyruleno  # Number of the rule by which to reduce
                  ):
        '''Perform a reduce action and the shift that must immediately
        follow the reduce.'''
        
        if (self.yyTraceFILE and
            yyruleno >= 0 and yyruleno < len(self.yyRuleName)
            ):
            self.yyTraceFILE.write("%sReduce [%s].\n" % (
                self.yyTracePrompt, self.yyRuleName[yyruleno]))

        # get the action
        action = self.yy_action_method[yyruleno]

        # 'yygotominor' is the LHS of the rule reduced
        yygotominor = action(self)

        yygoto = self.yyRuleInfo[yyruleno].lhs   # The next state
        yysize = self.yyRuleInfo[yyruleno].nrhs  # Amount to pop the stack
        if yysize > 0:
            del self.yystack[-yysize:]

        # The next action
        yyact = self.yy_find_reduce_action(self.yystack[-1].stateno, yygoto)

        if yyact < self.YYNSTATE:
            self.yy_shift(yyact, yygoto, yygotominor)
        else:
            assert yyact == self.YYNSTATE + self.YYNRULE + 1
            self.yy_accept()

        return


    def yy_parse_failed(self):
        '''This method executes when the parse fails.'''

        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sFail!\n" % self.yyTracePrompt)

        while self.yystack:
            self.yy_pop_parser_stack()

        self.delegate.parse_failed()

        return


    def yy_syntax_error(self, token):
        '''This method executes when a syntax error occurs.'''
        self.delegate.syntax_error(token)
        return


    def yy_accept(self):
        '''This method executes when the parser accepts.'''

        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sAccept!\n" % self.yyTracePrompt)

        while self.yystack:
            self.yy_pop_parser_stack()

        self.delegate.accept()

        return


    def parse(self, tokens):
        for token in tokens:
            self.feed(token)
        self.feed((0, None))
        return


    def feed(self, token):
        '''The main parser routine.'''

        yymajor = token[0]  # The major token code number
        yyminor = token[1]  # The value for the token

        yyerrorhit = False  # True if yymajor has invoked an error

        # (re)initialize the parser, if necessary
        if not self.yystack:
            self.yyerrcnt = -1
            yytos = self.yyStackEntry(
                stateno = 0,
                major = 0,
                minor = None
                )
            self.yystack.append(yytos)

        yyendofinput = (yymajor == 0) # True if we are at the end of input
        
        if self.yyTraceFILE:
            self.yyTraceFILE.write(
                "%sInput %s\n" %
                (self.yyTracePrompt, self.yyTokenName[yymajor]))


        cond = True
        while cond:

            # The parser action.
            yyact = self.yy_find_shift_action(yymajor)

            YYNOCODE = self.YYNOCODE
            YYNSTATE = self.YYNSTATE
            YYNRULE  = self.YYNRULE

            if yyact < YYNSTATE:
                assert not yyendofinput, "Impossible to shift the $ token"
                self.yy_shift(yyact, yymajor, yyminor)
                self.yyerrcnt -= 1
                yymajor = YYNOCODE
            elif yyact < YYNSTATE + YYNRULE:
                self.yy_reduce(yyact - YYNSTATE)
            else:
                assert yyact == self.YY_ERROR_ACTION
                if self.yyTraceFILE:
                    self.yyTraceFILE.write(
                        "%sSyntax Error!\n" % self.yyTracePrompt)

                YYERRORSYMBOL = self.YYERRORSYMBOL
                if YYERRORSYMBOL is not None:
                    # A syntax error has occurred.
                    # The response to an error depends upon whether or not the
                    # grammar defines an error token "ERROR".  
                    #
                    # This is what we do if the grammar does define ERROR:
                    #
                    #  * Call the %syntax_error function.
                    #
                    #  * Begin popping the stack until we enter a state where
                    #    it is legal to shift the error symbol, then shift
                    #    the error symbol.
                    #
                    #  * Set the error count to three.
                    #
                    #  * Begin accepting and shifting new tokens.  No new error
                    #    processing will occur until three tokens have been
                    #    shifted successfully.
                    #
                    if self.yyerrcnt < 0:
                        self.yy_syntax_error(token)

                    yymx = self.yystack[-1].major
                    if yymx == YYERRORSYMBOL or yyerrorhit:
                        if self.yyTraceFILE:
                            self.yyTraceFILE.write(
                                "%sDiscard input token %s\n" % (
                                    self.yyTracePrompt,
                                    self.yyTokenName[yymajor]))
                        yymajor = YYNOCODE
                    else:
                        while self.yystack and yymx != YYERRORSYMBOL:
                            yyact = self.yy_find_reduce_action(
                                self.yystack[-1].stateno,
                                YYERRORSYMBOL
                                )
                            if yyact < YYNSTATE:
                                break
                            self.yy_pop_parser_stack()

                        if not self.yystack or yymajor == 0:
                            self.yy_parse_failed()
                            yymajor = YYNOCODE
                        elif yymx != YYERRORSYMBOL:
                            self.yy_shift(yyact, YYERRORSYMBOL, None)

                    self.yyerrcnt = 3
                    yyerrorhit = True

                else: # YYERRORSYMBOL is not defined
                    # This is what we do if the grammar does not define ERROR:
                    #
                    #  * Report an error message, and throw away the input token.
                    #
                    #  * If the input token is $, then fail the parse.
                    #
                    # As before, subsequent error messages are suppressed until
                    # three input tokens have been successfully shifted.
                    #
                    if self.yyerrcnt <= 0:
                        self.yy_syntax_error(token)

                    self.yyerrcnt = 3
                    if yyendofinput:
                        self.yy_parse_failed()

                    yymajor = YYNOCODE

            cond = yymajor != YYNOCODE and self.yystack

        return


