# -*- Python -*-

# Driver template for the LEMON parser generator.
# The author disclaims copyright to this source code.


# First off, define the token values.  These constants (all generated
# automatically by the parser generator) specify the various kinds of
# tokens (terminals) that the parser understands.
#
# Each symbol here is a terminal symbol in the grammar.

T_SEMI                         =  1
L_NAME                         =  2
T_COLON                        =  3
K_DEFINE                       =  4
T_EQDEF                        =  5
K_ASSUME                       =  6
K_OBSERVE                      =  7
K_PREDICT                      =  8
K_FORGET                       =  9
K_FREEZE                       = 10
K_REPORT                       = 11
K_INFER                        = 12
K_CLEAR                        = 13
K_ROLLBACK                     = 14
K_LIST                         = 15
K_DIRECTIVES                   = 16
K_GET                          = 17
K_DIRECTIVE                    = 18
K_FORCE                        = 19
K_SAMPLE                       = 20
K_CONTINUOUS_INFERENCE_STATUS  = 21
K_START                        = 22
K_CONTINUOUS                   = 23
K_INFERENCE                    = 24
K_STOP_CONTINUOUS_INFERENCE    = 25
K_CURRENT                      = 26
K_EXCEPTION                    = 27
K_STATE                        = 28
K_GLOBAL                       = 29
K_LOGSCORE                     = 30
K_LOAD                         = 31
L_STRING                       = 32
L_INTEGER                      = 33
T_LARR                         = 34
K_AND                          = 35
T_AND                          = 36
K_OR                           = 37
T_OR                           = 38
K_EQ                           = 39
T_EQ                           = 40
K_NEQ                          = 41
T_NEQ                          = 42
K_LT                           = 43
T_LT                           = 44
K_LE                           = 45
T_LE                           = 46
K_GE                           = 47
T_GE                           = 48
K_GT                           = 49
T_GT                           = 50
K_ADD                          = 51
T_ADD                          = 52
K_SUB                          = 53
T_SUB                          = 54
K_MUL                          = 55
T_MUL                          = 56
K_DIV                          = 57
T_DIV                          = 58
K_POW                          = 59
T_POW                          = 60
T_LROUND                       = 61
T_RROUND                       = 62
T_COMMA                        = 63
T_LCURLY                       = 64
T_RCURLY                       = 65
K_PROC                         = 66
K_IF                           = 67
K_ELSE                         = 68
T_TRUE                         = 69
T_FALSE                        = 70
L_REAL                         = 71
L_TAG                          = 72
REAL                           = 73
T_LSQUARE                      = 74
T_RSQUARE                      = 75
T_STRING                       = 76
K_STATUS                       = 77
K_STOP                         = 78
K_LAMBDA                       = 79

# The state of the parser is completely contained in an instance of
# the following class.

class Parser(object):

    # defaults
    YYERRORSYMBOL = None
    YYWILDCARD = None

    # The next thing included is series of definitions which control
    # various aspects of the generated parser.
    #    YYNOCODE           is a number which corresponds
    #                       to no legal terminal or nonterminal number.  This
    #                       number is used to fill in empty slots of the hash 
    #                       table.
    #    YYNSTATE           the combined number of states.
    #    YYNRULE            the number of rules in the grammar
    #    YYERRORSYMBOL      is the code number of the error symbol.  If not
    #                       defined, then do no error processing.

    YYNOCODE = 114
    YYNSTATE = 194
    YYNRULE = 103
    YYERRORSYMBOL = 80
    YY_NO_ACTION     = YYNSTATE + YYNRULE + 2
    YY_ACCEPT_ACTION = YYNSTATE + YYNRULE + 1
    YY_ERROR_ACTION  = YYNSTATE + YYNRULE


    # Next are that tables used to determine what action to take based on the
    # current state and lookahead token.  These tables are used to implement
    # functions that take a state number and lookahead value and return an
    # action integer.  
    #
    # Suppose the action integer is N.  Then the action is determined as
    # follows
    #
    #   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
    #                                      token onto the stack and goto state N.
    #
    #   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
    #
    #   N == YYNSTATE+YYNRULE              A syntax error has occurred.
    #
    #   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
    #
    #   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
    #                                      slots in the yy_action[] table.
    #
    # The action table is constructed as a single large table named yy_action[].
    # Given state S and lookahead X, the action is computed as
    #
    #      yy_action[ yy_shift_ofst[S] + X ]
    #
    # If the index value yy_shift_ofst[S]+X is out of range or if the value
    # yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
    # is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
    # and that yy_default[S] should be used instead.  
    #
    # The formula above is for computing the action when the lookahead is
    # a terminal symbol.  If the lookahead is a non-terminal (as occurs after
    # a reduce action) then the yy_reduce_ofst[] array is used in place of
    # the yy_shift_ofst[] array and YY_REDUCE_USE_DFLT is used in place of
    # YY_SHIFT_USE_DFLT.
    #
    # The following are the tables generated in this section:
    #
    #  yy_action[]        A single table containing all actions.
    #  yy_lookahead[]     A table containing the lookahead for each entry in
    #                     yy_action.  Used to detect hash collisions.
    #  yy_shift_ofst[]    For each state, the offset into yy_action for
    #                     shifting terminals.
    #  yy_reduce_ofst[]   For each state, the offset into yy_action for
    #                     shifting non-terminals after a reduce.
    #  yy_default[]       Default action for each state.

    YY_ACTTAB_COUNT = 589
    yy_action = [
          194,   17,   74,   18,  113,   36,   84,   14,   13,   50, #     0
           49,   48,   12,  127,  126,   81,   47,   56,  193,   11, #    10
           10,  118,   77,  112,   80,  116,  122,   79,   68,   46, #    20
           18,   75,  170,  172,  114,  192,  136,  135,  134,  175, #    30
           23,   23,   22,   22,  189,   73,   70,   63,   53,   58, #    40
           59,  179,   67,   19,  185,  148,  164,  163,   52,    9, #    50
           18,    6,   20,   20,   43,   40,  108,  104,    8,  174, #    60
          173,  171,   34,  178,  175,   42,   87,  110,  180,  189, #    70
           73,   70,   63,   53,   58,   59,  179,   67,    5,  185, #    80
          178,  175,  106,  105,  110,  180,  189,   73,   70,   63, #    90
           53,   58,   59,  179,   67,  177,  185,  178,  175,   92, #   100
          103,  110,  180,  189,   73,   70,   63,   53,   58,   59, #   110
          179,   67,    4,  185,  178,  175,  102,  101,  110,  180, #   120
          189,   73,   70,   63,   53,   58,   59,  179,   67,    3, #   130
          185,  155,  178,  175,  100,   98,  110,  180,  189,   73, #   140
           70,   63,   53,   58,   59,  179,   67,   99,  185,  140, #   150
          175,   32,   32,    7,    2,  189,   73,   70,   63,   53, #   160
           58,   59,  179,   67,   86,  185,   85,  184,  175,  176, #   170
           21,   21,  183,  189,   73,   70,   63,   53,   58,   59, #   180
          179,   67,  169,  185,   89,  175,   41,   35,  157,  183, #   190
          189,   73,   70,   63,   53,   58,   59,  179,   67,  150, #   200
          185,  190,  175,  168,   38,  160,   92,  189,   73,   70, #   210
           63,   53,   58,   59,  179,   67,  161,  185,  188,  175, #   220
           29,   29,   28,   28,  189,   73,   70,   63,   53,   58, #   230
           59,  179,   67,  156,  185,   37,  182,  175,  151,  131, #   240
          158,   36,  189,   73,   70,   63,   53,   58,   59,  179, #   250
           67,   93,  185,  141,  175,  109,  180,  107,   91,  189, #   260
           73,   70,   63,   53,   58,   59,  179,   67,   90,  185, #   270
          132,  139,  175,  298,    1,  147,   69,  189,   73,   70, #   280
           63,   53,   58,   59,  179,   67,  144,  185,   82,  175, #   290
           17,  143,  142,   83,  189,   73,   70,   63,   53,   58, #   300
           59,  179,   67,   16,  185,  137,  175,   51,  152,   15, #   310
           45,  189,   73,   70,   63,   53,   58,   59,  179,   67, #   320
          123,  185,  128,  175,  125,  121,   44,  117,  189,   73, #   330
           70,   63,   53,   58,   59,  179,   67,   76,  185,  115, #   340
           78,  175,   95,   94,  145,  133,  189,   73,   70,   63, #   350
           53,   58,   59,  179,   67,  130,  185,  119,  175,  111, #   360
          129,  124,  191,  189,   73,   70,   63,   53,   58,   59, #   370
          179,   67,  175,  185,  170,  172,  138,  120,   88,   72, #   380
           63,   53,   58,   59,  179,   67,  146,  185,   93,  170, #   390
          172,   27,   27,   26,   26,   25,   25,   24,   24,  299, #   400
          299,  299,  299,    6,  170,  172,   43,  299,  108,  104, #   410
          299,  174,  173,  171,   34,   71,  299,  299,    6,  299, #   420
          299,   43,  175,  108,  104,  299,  174,  173,  171,   34, #   430
           66,   53,   58,   59,  179,   67,  299,  185,  175,  299, #   440
          299,  174,  173,  171,   34,  170,  172,   55,   58,   59, #   450
          179,   67,  175,  185,  167,  166,  299,  299,  175,  299, #   460
          299,   54,   58,   59,  179,   67,  299,  185,   65,   59, #   470
          179,   67,  299,  185,    6,  299,  299,   43,  175,  108, #   480
          104,  299,  174,  173,  171,   34,   39,  299,   62,   59, #   490
          179,   67,  175,  185,  299,  165,   30,  162,  175,  299, #   500
          167,  166,   60,   59,  179,   67,  299,  185,   57,   59, #   510
          179,   67,  175,  185,   33,   33,   31,   31,  175,   96, #   520
          299,  299,  299,   64,  179,   67,  299,  185,  299,   61, #   530
          179,   67,   39,  185,  113,  299,   84,   14,   13,  175, #   540
          299,  165,   30,  175,  299,  299,   97,  164,  163,  299, #   550
          175,  187,   67,  299,  185,  186,   67,  299,  185,  159, #   560
          164,  163,  181,   67,  299,  185,  299,  299,  299,  299, #   570
          154,  164,  163,  153,  164,  163,  149,  164,  163, #   580
        ]
    yy_lookahead = [
            0,    5,    2,   34,    4,    3,    6,    7,    8,    9, #     0
           10,   11,   12,   13,   14,   15,   18,   17,    1,   19, #    10
           20,   21,   22,    2,   26,   25,   28,   29,   80,    3, #    20
           34,   31,   32,   33,   83,   84,   85,   86,   87,   88, #    30
           39,   40,   41,   42,   93,   94,   95,   96,   97,   98, #    40
           99,  100,  101,    5,  103,  107,  108,  109,  110,    1, #    50
           34,   61,   35,   36,   64,   63,   66,   67,    1,   69, #    60
           70,   71,   72,   87,   88,   61,   90,   91,   92,   93, #    70
           94,   95,   96,   97,   98,   99,  100,  101,   64,  103, #    80
           87,   88,   62,   90,   91,   92,   93,   94,   95,   96, #    90
           97,   98,   99,  100,  101,   65,  103,   87,   88,   80, #   100
           90,   91,   92,   93,   94,   95,   96,   97,   98,   99, #   110
          100,  101,   61,  103,   87,   88,   62,   90,   91,   92, #   120
           93,   94,   95,   96,   97,   98,   99,  100,  101,   64, #   130
          103,  112,   87,   88,   65,   90,   91,   92,   93,   94, #   140
           95,   96,   97,   98,   99,  100,  101,   68,  103,   87, #   150
           88,   59,   60,   61,   64,   93,   94,   95,   96,   97, #   160
           98,   99,  100,  101,  102,  103,  104,   87,   88,   65, #   170
           37,   38,   92,   93,   94,   95,   96,   97,   98,   99, #   180
          100,  101,   50,  103,   87,   88,   63,   63,   65,   92, #   190
           93,   94,   95,   96,   97,   98,   99,  100,  101,   75, #   200
          103,   87,   88,   50,   63,   75,   80,   93,   94,   95, #   210
           96,   97,   98,   99,  100,  101,   75,  103,   87,   88, #   220
           51,   52,   53,   54,   93,   94,   95,   96,   97,   98, #   230
           99,  100,  101,   65,  103,    3,   87,   88,  112,    2, #   240
           65,    3,   93,   94,   95,   96,   97,   98,   99,  100, #   250
          101,   76,  103,   87,   88,   91,   92,  105,  106,   93, #   260
           94,   95,   96,   97,   98,   99,  100,  101,   63,  103, #   270
           33,   87,   88,   81,   82,    2,   80,   93,   94,   95, #   280
           96,   97,   98,   99,  100,  101,   65,  103,   87,   88, #   290
            5,   62,   62,    2,   93,   94,   95,   96,   97,   98, #   300
           99,  100,  101,   63,  103,   87,   88,  111,  112,    5, #   310
            5,   93,   94,   95,   96,   97,   98,   99,  100,  101, #   320
           27,  103,   87,   88,   16,   30,    5,   24,   93,   94, #   330
           95,   96,   97,   98,   99,  100,  101,   23,  103,   32, #   340
           87,   88,   80,   80,    2,   89,   93,   94,   95,   96, #   350
           97,   98,   99,  100,  101,   89,  103,   87,   88,    2, #   360
           89,   89,   85,   93,   94,   95,   96,   97,   98,   99, #   370
          100,  101,   88,  103,   32,   33,   88,   88,    2,   95, #   380
           96,   97,   98,   99,  100,  101,    2,  103,   76,   32, #   390
           33,   43,   44,   45,   46,   47,   48,   49,   50,  113, #   400
          113,  113,  113,   61,   32,   33,   64,  113,   66,   67, #   410
          113,   69,   70,   71,   72,    2,  113,  113,   61,  113, #   420
          113,   64,   88,   66,   67,  113,   69,   70,   71,   72, #   430
           96,   97,   98,   99,  100,  101,  113,  103,   88,  113, #   440
          113,   69,   70,   71,   72,   32,   33,   97,   98,   99, #   450
          100,  101,   88,  103,   32,   33,  113,  113,   88,  113, #   460
          113,   97,   98,   99,  100,  101,  113,  103,   98,   99, #   470
          100,  101,  113,  103,   61,  113,  113,   64,   88,   66, #   480
           67,  113,   69,   70,   71,   72,   64,  113,   98,   99, #   490
          100,  101,   88,  103,  113,   73,   74,   75,   88,  113, #   500
           32,   33,   98,   99,  100,  101,  113,  103,   98,   99, #   510
          100,  101,   88,  103,   55,   56,   57,   58,   88,   80, #   520
          113,  113,  113,   99,  100,  101,  113,  103,  113,   99, #   530
          100,  101,   64,  103,    4,  113,    6,    7,    8,   88, #   540
          113,   73,   74,   88,  113,  113,  107,  108,  109,  113, #   550
           88,  100,  101,  113,  103,  100,  101,  113,  103,  107, #   560
          108,  109,  100,  101,  113,  103,  113,  113,  113,  113, #   570
          107,  108,  109,  107,  108,  109,  107,  108,  109, #   580
        ]
    YY_SHIFT_USE_DFLT = -32
    YY_SHIFT_COUNT = 114
    YY_SHIFT_MIN = -31
    YY_SHIFT_MAX = 540
    yy_shift_ofst = [
          -32,    0,  423,  423,  423,  423,  423,  367,  423,  423, #     0
          367,  367,  367,  367,  367,  367,  367,  367,  367,  367, #    10
          352,  352,  352,  352,  352,  352,  352,  352,  352,  352, #    20
          432,  352,  352,  352,  478,  478,  478,  478,  478,  185, #    30
          322,  322,  394,  386,  382,  382,  540,  247,  247,  247, #    40
          247,  133,  151,  358,  358,  358,   -2,  179,  179,  469, #    50
          179,  469,  179,    1,  469,  179,    1,  102,  134,    2, #    60
          143,   -4,  143,   27,   26,  317,  313,  324,  331,  305, #    70
          303,  318,  315,  314,  301,  250,  240,  239,  295,  231, #    80
          283,  215,  248,  242,  178,  140,  163,  142,  114,  100, #    90
           89,   79,   75,   64,   61,   40,   24,   30,   14,   67, #   100
           58,  -31,   48,   21,   17, #   110
        ]
    YY_REDUCE_USE_DFLT = -53
    YY_REDUCE_COUNT = 52
    YY_REDUCE_MIN = -52
    YY_REDUCE_MAX = 479
    yy_reduce_ofst = [
          202,  -49,   55,   37,   20,    3,  -14,   72,  107,   90, #     0
          280,  263,  245,  228,  211,  194,  176,  159,  141,  124, #    10
          294,  344,  374,  360,  420,  414,  400,  380,  440,  434, #    20
          -52,  472,  465,  461,  449,  479,  476,  473,  462,  206, #    30
          136,   29,  162,  174,  299,  298,  287,  282,  281,  276, #    40
          266,  273,  272, #    50
        ]
    yy_default = [
          195,  197,  297,  297,  297,  297,  297,  256,  297,  297, #     0
          297,  297,  297,  297,  297,  297,  297,  297,  297,  297, #    10
          297,  297,  297,  297,  297,  297,  297,  297,  297,  297, #    20
          297,  297,  297,  297,  297,  297,  297,  297,  297,  297, #    30
          297,  297,  266,  297,  297,  297,  297,  297,  297,  297, #    40
          297,  297,  297,  240,  239,  238,  297,  244,  245,  248, #    50
          243,  247,  242,  237,  246,  241,  236,  253,  297,  297, #    60
          235,  265,  234,  233,  265,  297,  297,  297,  297,  297, #    70
          297,  297,  297,  297,  297,  257,  297,  297,  297,  297, #    80
          297,  267,  297,  297,  297,  297,  297,  297,  297,  297, #    90
          297,  297,  297,  297,  297,  297,  297,  297,  297,  297, #   100
          297,  265,  297,  297,  297,  223,  219,  218,  217,  216, #   110
          215,  222,  221,  220,  214,  213,  212,  211,  210,  209, #   120
          208,  225,  224,  207,  202,  201,  200,  206,  205,  204, #   130
          258,  259,  254,  260,  261,  265,  268,  269,  286,  288, #   140
          285,  294,  292,  296,  295,  293,  291,  290,  289,  287, #   150
          284,  283,  282,  281,  280,  279,  278,  277,  276,  275, #   160
          274,  273,  272,  271,  270,  264,  263,  262,  227,  251, #   170
          228,  250,  230,  229,  226,  255,  252,  249,  232,  231, #   180
          203,  199,  198,  196, #   190
        ]


    # The next table maps tokens into fallback tokens.  If a construct
    # like the following:
    #
    #      %fallback ID X Y Z.
    #
    # appears in the grammer, then ID becomes a fallback token for X, Y,
    # and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
    # but it does not parse, the type of the token is changed to ID and
    # the parse is retried before an error is thrown.

    yyFallback = [
          0,  #          $ => nothing
          0,  #     T_SEMI => nothing
          0,  #     L_NAME => nothing
          0,  #    T_COLON => nothing
          2,  #   K_DEFINE => L_NAME
          0,  #    T_EQDEF => nothing
          2,  #   K_ASSUME => L_NAME
          2,  #  K_OBSERVE => L_NAME
          2,  #  K_PREDICT => L_NAME
          2,  #   K_FORGET => L_NAME
          2,  #   K_FREEZE => L_NAME
          2,  #   K_REPORT => L_NAME
          2,  #    K_INFER => L_NAME
          2,  #    K_CLEAR => L_NAME
          2,  # K_ROLLBACK => L_NAME
          2,  #     K_LIST => L_NAME
          2,  # K_DIRECTIVES => L_NAME
          2,  #      K_GET => L_NAME
          2,  # K_DIRECTIVE => L_NAME
          2,  #    K_FORCE => L_NAME
          2,  #   K_SAMPLE => L_NAME
          0,  # K_CONTINUOUS_INFERENCE_STATUS => nothing
          2,  #    K_START => L_NAME
          2,  # K_CONTINUOUS => L_NAME
          2,  # K_INFERENCE => L_NAME
          0,  # K_STOP_CONTINUOUS_INFERENCE => nothing
          2,  #  K_CURRENT => L_NAME
          2,  # K_EXCEPTION => L_NAME
          2,  #    K_STATE => L_NAME
          2,  #   K_GLOBAL => L_NAME
          2,  # K_LOGSCORE => L_NAME
          2,  #     K_LOAD => L_NAME
          0,  #   L_STRING => nothing
          0,  #  L_INTEGER => nothing
          0,  #     T_LARR => nothing
          2,  #      K_AND => L_NAME
          0,  #      T_AND => nothing
          2,  #       K_OR => L_NAME
          0,  #       T_OR => nothing
          2,  #       K_EQ => L_NAME
          0,  #       T_EQ => nothing
          2,  #      K_NEQ => L_NAME
          0,  #      T_NEQ => nothing
          2,  #       K_LT => L_NAME
          0,  #       T_LT => nothing
          2,  #       K_LE => L_NAME
          0,  #       T_LE => nothing
          2,  #       K_GE => L_NAME
          0,  #       T_GE => nothing
          2,  #       K_GT => L_NAME
          0,  #       T_GT => nothing
          2,  #      K_ADD => L_NAME
          0,  #      T_ADD => nothing
          2,  #      K_SUB => L_NAME
          0,  #      T_SUB => nothing
          2,  #      K_MUL => L_NAME
          0,  #      T_MUL => nothing
          2,  #      K_DIV => L_NAME
          0,  #      T_DIV => nothing
          2,  #      K_POW => L_NAME
          0,  #      T_POW => nothing
          0,  #   T_LROUND => nothing
          0,  #   T_RROUND => nothing
          0,  #    T_COMMA => nothing
          0,  #   T_LCURLY => nothing
          0,  #   T_RCURLY => nothing
          2,  #     K_PROC => L_NAME
          2,  #       K_IF => L_NAME
          2,  #     K_ELSE => L_NAME
          0,  #     T_TRUE => nothing
          0,  #    T_FALSE => nothing
          0,  #     L_REAL => nothing
          0,  #      L_TAG => nothing
          0,  #       REAL => nothing
          0,  #  T_LSQUARE => nothing
          0,  #  T_RSQUARE => nothing
          0,  #   T_STRING => nothing
          2,  #   K_STATUS => L_NAME
          2,  #     K_STOP => L_NAME
        ]


    # The following structure represents a single element of the
    # parser's stack.  Information stored includes:
    #
    #   +  The state number for the parser at this level of the stack.
    #
    #   +  The value of the token stored at this level of the stack.
    #      (In other words, the "major" token.)
    #
    #   +  The semantic value stored at this level of the stack.  This is
    #      the information used by the action routines in the grammar.
    #      It is sometimes called the "minor" token.
    #
    class yyStackEntry(object):
        def __init__(
            self,
            stateno, # The state-number
            major,   # The major token value.  This is the code
                     # number for the token at this stack level
            minor,   # The user-supplied minor token value.  This
                     # is the value of the token
            ):
            self.stateno = stateno
            self.major = major
            self.minor = minor
            return


    yyTraceFILE = None
    yyTracePrompt = None

    def trace(self, TraceFILE, zTracePrompt):
        '''Turn parser tracing on by giving a stream to which to write
        the trace and a prompt to preface each trace message.  Tracing
        is turned off by making either argument None.
        '''
        self.yyTraceFILE = TraceFILE
        self.yyTracePrompt = zTracePrompt
        if self.yyTraceFILE is None:
            self.yyTracePrompt = None
        elif self.yyTracePrompt is None:
            self.yyTraceFILE = None
        return


    # For tracing shifts, the names of all terminals and nonterminals
    # are required.  The following table supplies these names
    yyTokenName = [
        "$",                   "T_SEMI",              "L_NAME",              "T_COLON",     
        "K_DEFINE",            "T_EQDEF",             "K_ASSUME",            "K_OBSERVE",   
        "K_PREDICT",           "K_FORGET",            "K_FREEZE",            "K_REPORT",    
        "K_INFER",             "K_CLEAR",             "K_ROLLBACK",          "K_LIST",      
        "K_DIRECTIVES",        "K_GET",               "K_DIRECTIVE",         "K_FORCE",     
        "K_SAMPLE",            "K_CONTINUOUS_INFERENCE_STATUS",        "K_START",             "K_CONTINUOUS",
        "K_INFERENCE",         "K_STOP_CONTINUOUS_INFERENCE",        "K_CURRENT",           "K_EXCEPTION", 
        "K_STATE",             "K_GLOBAL",            "K_LOGSCORE",          "K_LOAD",      
        "L_STRING",            "L_INTEGER",           "T_LARR",              "K_AND",       
        "T_AND",               "K_OR",                "T_OR",                "K_EQ",        
        "T_EQ",                "K_NEQ",               "T_NEQ",               "K_LT",        
        "T_LT",                "K_LE",                "T_LE",                "K_GE",        
        "T_GE",                "K_GT",                "T_GT",                "K_ADD",       
        "T_ADD",               "K_SUB",               "T_SUB",               "K_MUL",       
        "T_MUL",               "K_DIV",               "T_DIV",               "K_POW",       
        "T_POW",               "T_LROUND",            "T_RROUND",            "T_COMMA",     
        "T_LCURLY",            "T_RCURLY",            "K_PROC",              "K_IF",        
        "K_ELSE",              "T_TRUE",              "T_FALSE",             "L_REAL",      
        "L_TAG",               "REAL",                "T_LSQUARE",           "T_RSQUARE",   
        "T_STRING",            "K_STATUS",            "K_STOP",              "K_LAMBDA",    
        "error",               "venture",             "instructions",        "instruction_opt",
        "instruction",         "directive",           "command",             "expression",  
        "literal",             "directive_ref",        "body",                "let",         
        "let1",                "do_bind",             "boolean_and",         "boolean_or",  
        "equality",            "comparison",          "additive",            "multiplicative",
        "exponential",         "applicative",         "arglist",             "primary",     
        "args",                "paramlist",           "params",              "json",        
        "json_list",           "json_dict",           "json_list_terms",        "json_dict_entries",
        "json_dict_entry",
        ]

    # For tracing reduce actions, the names of all rules are required.
    yyRuleName = [
        "venture ::= instructions", #   0
        "instructions ::=", #   1
        "instructions ::= instructions instruction_opt T_SEMI", #   2
        "instruction_opt ::=", #   3
        "instruction_opt ::= instruction", #   4
        "instruction ::= L_NAME T_COLON directive", #   5
        "instruction ::= directive", #   6
        "instruction ::= command", #   7
        "instruction ::= expression", #   8
        "directive ::= K_DEFINE L_NAME T_EQDEF expression", #   9
        "directive ::= K_ASSUME L_NAME T_EQDEF expression", #  10
        "directive ::= K_OBSERVE expression T_EQDEF literal", #  11
        "directive ::= K_PREDICT expression", #  12
        "command ::= K_FORGET directive_ref", #  13
        "command ::= K_FREEZE directive_ref", #  14
        "command ::= K_REPORT directive_ref", #  15
        "command ::= K_INFER expression", #  16
        "command ::= K_CLEAR", #  17
        "command ::= K_ROLLBACK", #  18
        "command ::= K_LIST K_DIRECTIVES", #  19
        "command ::= K_GET K_DIRECTIVE directive_ref", #  20
        "command ::= K_FORCE expression T_EQDEF literal", #  21
        "command ::= K_SAMPLE expression", #  22
        "command ::= K_CONTINUOUS_INFERENCE_STATUS", #  23
        "command ::= K_START K_CONTINUOUS K_INFERENCE", #  24
        "command ::= K_STOP_CONTINUOUS_INFERENCE", #  25
        "command ::= K_GET K_CURRENT K_EXCEPTION", #  26
        "command ::= K_GET K_STATE", #  27
        "command ::= K_GET K_GLOBAL K_LOGSCORE", #  28
        "command ::= K_LOAD L_STRING", #  29
        "directive_ref ::= L_INTEGER", #  30
        "directive_ref ::= L_NAME", #  31
        "body ::= let T_SEMI expression", #  32
        "body ::= expression", #  33
        "let ::= let1", #  34
        "let ::= let T_SEMI let1", #  35
        "let1 ::= L_NAME T_EQDEF expression", #  36
        "expression ::= do_bind", #  37
        "do_bind ::= L_NAME T_LARR expression", #  38
        "do_bind ::= boolean_and", #  39
        "boolean_and ::= boolean_and K_AND|T_AND boolean_or", #  40
        "boolean_and ::= boolean_or", #  41
        "boolean_or ::= boolean_or K_OR|T_OR equality", #  42
        "boolean_or ::= equality", #  43
        "equality ::= equality K_EQ|T_EQ comparison", #  44
        "equality ::= equality K_NEQ|T_NEQ comparison", #  45
        "equality ::= comparison", #  46
        "comparison ::= comparison K_LT|T_LT additive", #  47
        "comparison ::= comparison K_LE|T_LE additive", #  48
        "comparison ::= comparison K_GE|T_GE additive", #  49
        "comparison ::= comparison K_GT|T_GT additive", #  50
        "comparison ::= additive", #  51
        "additive ::= additive K_ADD|T_ADD multiplicative", #  52
        "additive ::= additive K_SUB|T_SUB multiplicative", #  53
        "additive ::= multiplicative", #  54
        "multiplicative ::= multiplicative K_MUL|T_MUL exponential", #  55
        "multiplicative ::= multiplicative K_DIV|T_DIV exponential", #  56
        "multiplicative ::= exponential", #  57
        "exponential ::= applicative K_POW|T_POW exponential", #  58
        "exponential ::= applicative", #  59
        "applicative ::= applicative T_LROUND arglist T_RROUND", #  60
        "applicative ::= primary", #  61
        "arglist ::=", #  62
        "arglist ::= args", #  63
        "args ::= expression", #  64
        "args ::= args T_COMMA expression", #  65
        "primary ::= T_LROUND body T_RROUND", #  66
        "primary ::= T_LCURLY let T_SEMI expression T_RCURLY", #  67
        "primary ::= K_PROC T_LROUND paramlist T_RROUND T_LCURLY body T_RCURLY", #  68
        "primary ::= K_IF T_LROUND body T_RROUND T_LCURLY body T_RCURLY K_ELSE T_LCURLY body T_RCURLY", #  69
        "primary ::= literal", #  70
        "primary ::= L_NAME", #  71
        "paramlist ::=", #  72
        "paramlist ::= params", #  73
        "params ::= L_NAME", #  74
        "params ::= params T_COMMA L_NAME", #  75
        "literal ::= T_TRUE", #  76
        "literal ::= T_FALSE", #  77
        "literal ::= L_INTEGER", #  78
        "literal ::= L_REAL", #  79
        "literal ::= L_STRING", #  80
        "literal ::= L_TAG json T_GT", #  81
        "literal ::= L_TAG error T_GT", #  82
        "json ::= L_STRING", #  83
        "json ::= L_INTEGER", #  84
        "json ::= REAL", #  85
        "json ::= json_list", #  86
        "json ::= json_dict", #  87
        "json_list ::= T_LSQUARE T_RSQUARE", #  88
        "json_list ::= T_LSQUARE json_list_terms T_RSQUARE", #  89
        "json_list ::= T_LSQUARE json_list_terms error T_RSQUARE", #  90
        "json_list ::= T_LSQUARE error T_RSQUARE", #  91
        "json_list_terms ::= json", #  92
        "json_list_terms ::= json_list_terms T_COMMA json", #  93
        "json_list_terms ::= error T_COMMA json", #  94
        "json_dict ::= T_LCURLY T_RCURLY", #  95
        "json_dict ::= T_LCURLY json_dict_entries T_RCURLY", #  96
        "json_dict ::= T_LCURLY json_dict_entries error T_RCURLY", #  97
        "json_dict_entries ::= json_dict_entry", #  98
        "json_dict_entries ::= json_dict_entries T_COMMA json_dict_entry", #  99
        "json_dict_entries ::= error T_COMMA json_dict_entry", # 100
        "json_dict_entry ::= T_STRING T_COLON json", # 101
        "json_dict_entry ::= error T_COLON json", # 102
        ]


    def __init__(self, delegate):
        self.yystack = [] # The parser's stack
        self.delegate = delegate
        return


    def yy_pop_parser_stack(self):
        """Pop the parser's stack once. Return the major token number
        for the symbol popped.
        """
        if not self.yystack:
            return 0
        yytos = self.yystack.pop()
        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sPopping %s\n" % (
                self.yyTracePrompt,
                self.yyTokenName[yytos.major]))
        yymajor = yytos.major
        return yymajor


    def yy_find_shift_action(self,       # The parser
                             iLookAhead  # The look-ahead token
                             ):
        '''Find the appropriate action for a parser given the terminal
        look-ahead token iLookAhead.

        If the look-ahead token is YYNOCODE, then check to see if the
        action is independent of the look-ahead.  If it is, return the
        action, otherwise return YY_NO_ACTION.
        '''
        yyTraceFILE = self.yyTraceFILE
        stateno = self.yystack[-1].stateno
        if stateno > self.YY_SHIFT_COUNT:
            return self.yy_default[stateno]
        i = self.yy_shift_ofst[stateno]
        if i == self.YY_SHIFT_USE_DFLT:
            return self.yy_default[stateno]
        assert iLookAhead != self.YYNOCODE
        i += iLookAhead
        if i < 0 or i >= self.YY_ACTTAB_COUNT or self.yy_lookahead[i] != iLookAhead:
            if iLookAhead > 0:
                yyFallback = self.yyFallback
                yyTokenName = self.yyTokenName
                if iLookAhead < len(yyFallback):
                    iFallback = yyFallback[iLookAhead] # Fallback token
                    if iFallback != 0:
                        if yyTraceFILE:
                            yyTraceFILE.write(
                                "%sFALLBACK %s => %s\n" %
                                (self.yyTracePrompt,
                                 yyTokenName[iLookAhead], yyTokenName[iFallback]))
                        return self.yy_find_shift_action(iFallback);
                YYWILDCARD = self.YYWILDCARD
                if YYWILDCARD is not None:
                    j = i - iLookAhead + YYWILDCARD
                    # [TRC 20150122: Lemon avoids generating tests
                    # against j if they can never fail, based on
                    # compile-time values here, presumably to limit
                    # compiler warnings.  Not so easy to do in Python
                    # so we'll just skip that.]
                    if j >= 0 and j < self.YY_ACTTAB_COUNT and self.yy_lookahead[j] == YYWILDCARD:
                        if yyTraceFILE:
                            yyTraceFILE.write(
                                "%sWILDCARD %s => %s\n" %
                                (self.yyTracePrompt,
                                 yyTokenName[iLookAhead], yyTokenName[YYWILDCARD]))
                        return self.yy_action[j];
            return self.yy_default[stateno]
        else:
            return self.yy_action[i]


    def yy_find_reduce_action(self,
                              stateno,    # Current state number
                              iLookAhead  # The look-ahead token
                              ):
        '''Find the appropriate action for a parser given the
        non-terminal look-ahead token iLookAhead.
        
        If the look-ahead token is YYNOCODE, then check to see if the
        action is independent of the look-ahead.  If it is, return the
        action, otherwise return YY_NO_ACTION.
        '''
        YYERRORSYMBOL = self.YYERRORSYMBOL
        if YYERRORSYMBOL is not None:
            if stateno > self.YY_REDUCE_COUNT:
                return self.yy_default[stateno]
        else:
            assert stateno <= self.YY_REDUCE_COUNT
        i = self.yy_reduce_ofst[stateno]
        assert i != self.YY_REDUCE_USE_DFLT
        assert iLookAhead != self.YYNOCODE
        i += iLookAhead
        if YYERRORSYMBOL is not None:
            if i < 0 or i >= self.YY_ACTTAB_COUNT or self.yy_lookahead[i] != iLookAhead:
                return self.yy_default[stateno]
        else:
            assert i >= 0 and i < self.YY_ACTTAB_COUNT
            assert self.yy_lookahead[i] == iLookAhead
        return self.yy_action[i]


    def yy_shift(self,        # The parser to be shifted
                 yyNewState,  # The new state to shift in
                 yyMajor,     # The major token to shift in
                 yyMinor      # The minor token to shift in
                 ):
        '''Perform a shift action.'''

        yytos = self.yyStackEntry(
            stateno = yyNewState,
            major = yyMajor,
            minor = yyMinor
            )
        self.yystack.append(yytos)

        yyTraceFILE = self.yyTraceFILE
        if yyTraceFILE:
            yyTraceFILE.write("%sShift %d\n" % (self.yyTracePrompt, yyNewState))
            yyTraceFILE.write("%sStack:" % self.yyTracePrompt)
            for entry in self.yystack:
                yyTraceFILE.write(" %s" % self.yyTokenName[entry.major])
            yyTraceFILE.write("\n")

        return


    # The following table contains information about every rule that
    # is used during the reduce.
    from collections import namedtuple
    yyRuleInfoEntry = namedtuple(
        'yyRuleInfoEntry',
        ('lhs',  # Symbol on the left-hand side of the rule
         'nrhs', # Number of right-hand side symbols in the rule
         ))
    yyRuleInfo = [
        yyRuleInfoEntry( 81, 1 ),
        yyRuleInfoEntry( 82, 0 ),
        yyRuleInfoEntry( 82, 3 ),
        yyRuleInfoEntry( 83, 0 ),
        yyRuleInfoEntry( 83, 1 ),
        yyRuleInfoEntry( 84, 3 ),
        yyRuleInfoEntry( 84, 1 ),
        yyRuleInfoEntry( 84, 1 ),
        yyRuleInfoEntry( 84, 1 ),
        yyRuleInfoEntry( 85, 4 ),
        yyRuleInfoEntry( 85, 4 ),
        yyRuleInfoEntry( 85, 4 ),
        yyRuleInfoEntry( 85, 2 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 1 ),
        yyRuleInfoEntry( 86, 1 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 3 ),
        yyRuleInfoEntry( 86, 4 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 1 ),
        yyRuleInfoEntry( 86, 3 ),
        yyRuleInfoEntry( 86, 1 ),
        yyRuleInfoEntry( 86, 3 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 86, 3 ),
        yyRuleInfoEntry( 86, 2 ),
        yyRuleInfoEntry( 89, 1 ),
        yyRuleInfoEntry( 89, 1 ),
        yyRuleInfoEntry( 90, 3 ),
        yyRuleInfoEntry( 90, 1 ),
        yyRuleInfoEntry( 91, 1 ),
        yyRuleInfoEntry( 91, 3 ),
        yyRuleInfoEntry( 92, 3 ),
        yyRuleInfoEntry( 87, 1 ),
        yyRuleInfoEntry( 93, 3 ),
        yyRuleInfoEntry( 93, 1 ),
        yyRuleInfoEntry( 94, 3 ),
        yyRuleInfoEntry( 94, 1 ),
        yyRuleInfoEntry( 95, 3 ),
        yyRuleInfoEntry( 95, 1 ),
        yyRuleInfoEntry( 96, 3 ),
        yyRuleInfoEntry( 96, 3 ),
        yyRuleInfoEntry( 96, 1 ),
        yyRuleInfoEntry( 97, 3 ),
        yyRuleInfoEntry( 97, 3 ),
        yyRuleInfoEntry( 97, 3 ),
        yyRuleInfoEntry( 97, 3 ),
        yyRuleInfoEntry( 97, 1 ),
        yyRuleInfoEntry( 98, 3 ),
        yyRuleInfoEntry( 98, 3 ),
        yyRuleInfoEntry( 98, 1 ),
        yyRuleInfoEntry( 99, 3 ),
        yyRuleInfoEntry( 99, 3 ),
        yyRuleInfoEntry( 99, 1 ),
        yyRuleInfoEntry( 100, 3 ),
        yyRuleInfoEntry( 100, 1 ),
        yyRuleInfoEntry( 101, 4 ),
        yyRuleInfoEntry( 101, 1 ),
        yyRuleInfoEntry( 102, 0 ),
        yyRuleInfoEntry( 102, 1 ),
        yyRuleInfoEntry( 104, 1 ),
        yyRuleInfoEntry( 104, 3 ),
        yyRuleInfoEntry( 103, 3 ),
        yyRuleInfoEntry( 103, 5 ),
        yyRuleInfoEntry( 103, 7 ),
        yyRuleInfoEntry( 103, 11 ),
        yyRuleInfoEntry( 103, 1 ),
        yyRuleInfoEntry( 103, 1 ),
        yyRuleInfoEntry( 105, 0 ),
        yyRuleInfoEntry( 105, 1 ),
        yyRuleInfoEntry( 106, 1 ),
        yyRuleInfoEntry( 106, 3 ),
        yyRuleInfoEntry( 88, 1 ),
        yyRuleInfoEntry( 88, 1 ),
        yyRuleInfoEntry( 88, 1 ),
        yyRuleInfoEntry( 88, 1 ),
        yyRuleInfoEntry( 88, 1 ),
        yyRuleInfoEntry( 88, 3 ),
        yyRuleInfoEntry( 88, 3 ),
        yyRuleInfoEntry( 107, 1 ),
        yyRuleInfoEntry( 107, 1 ),
        yyRuleInfoEntry( 107, 1 ),
        yyRuleInfoEntry( 107, 1 ),
        yyRuleInfoEntry( 107, 1 ),
        yyRuleInfoEntry( 108, 2 ),
        yyRuleInfoEntry( 108, 3 ),
        yyRuleInfoEntry( 108, 4 ),
        yyRuleInfoEntry( 108, 3 ),
        yyRuleInfoEntry( 110, 1 ),
        yyRuleInfoEntry( 110, 3 ),
        yyRuleInfoEntry( 110, 3 ),
        yyRuleInfoEntry( 109, 2 ),
        yyRuleInfoEntry( 109, 3 ),
        yyRuleInfoEntry( 109, 4 ),
        yyRuleInfoEntry( 111, 1 ),
        yyRuleInfoEntry( 111, 3 ),
        yyRuleInfoEntry( 111, 3 ),
        yyRuleInfoEntry( 112, 3 ),
        yyRuleInfoEntry( 112, 3 ),
        ]


    # Action code for each rule follows.
    def action_000(self):
        # venture ::= instructions
        return self.delegate.p_venture_top(
            insts = self.yystack[-1].minor,
            )
    def action_001(self):
        # instructions ::=
        return self.delegate.p_instructions_none(
            )
    def action_002(self):
        # instructions ::= instructions instruction_opt T_SEMI
        return self.delegate.p_instructions_some(
            insts = self.yystack[-3].minor,
            inst = self.yystack[-2].minor,
            )
    def action_003(self):
        # instruction_opt ::=
        return self.delegate.p_instruction_opt_none(
            )
    def action_004(self):
        # instruction_opt ::= instruction
        return self.delegate.p_instruction_opt_some(
            inst = self.yystack[-1].minor,
            )
    def action_005(self):
        # instruction ::= L_NAME T_COLON directive
        return self.delegate.p_instruction_labelled(
            l = self.yystack[-3].minor,
            d = self.yystack[-1].minor,
            )
    def action_006(self):
        # instruction ::= directive
        return self.delegate.p_instruction_unlabelled(
            d = self.yystack[-1].minor,
            )
    def action_007(self):
        # instruction ::= command
        return self.delegate.p_instruction_command(
            c = self.yystack[-1].minor,
            )
    def action_008(self):
        # instruction ::= expression
        return self.delegate.p_instruction_expression(
            e = self.yystack[-1].minor,
            )
    def action_009(self):
        # directive ::= K_DEFINE L_NAME T_EQDEF expression
        return self.delegate.p_directive_define(
            k = self.yystack[-4].minor,
            n = self.yystack[-3].minor,
            eq = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_010(self):
        # directive ::= K_ASSUME L_NAME T_EQDEF expression
        return self.delegate.p_directive_assume(
            k = self.yystack[-4].minor,
            n = self.yystack[-3].minor,
            eq = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_011(self):
        # directive ::= K_OBSERVE expression T_EQDEF literal
        return self.delegate.p_directive_observe(
            k = self.yystack[-4].minor,
            e = self.yystack[-3].minor,
            eq = self.yystack[-2].minor,
            v = self.yystack[-1].minor,
            )
    def action_012(self):
        # directive ::= K_PREDICT expression
        return self.delegate.p_directive_predict(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_013(self):
        # command ::= K_FORGET directive_ref
        return self.delegate.p_command_forget(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_014(self):
        # command ::= K_FREEZE directive_ref
        return self.delegate.p_command_freeze(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_015(self):
        # command ::= K_REPORT directive_ref
        return self.delegate.p_command_report(
            k = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_016(self):
        # command ::= K_INFER expression
        return self.delegate.p_command_infer(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_017(self):
        # command ::= K_CLEAR
        return self.delegate.p_command_clear(
            k = self.yystack[-1].minor,
            )
    def action_018(self):
        # command ::= K_ROLLBACK
        return self.delegate.p_command_rollback(
            k = self.yystack[-1].minor,
            )
    def action_019(self):
        # command ::= K_LIST K_DIRECTIVES
        return self.delegate.p_command_list_directives(
            k0 = self.yystack[-2].minor,
            k1 = self.yystack[-1].minor,
            )
    def action_020(self):
        # command ::= K_GET K_DIRECTIVE directive_ref
        return self.delegate.p_command_get_directive(
            k0 = self.yystack[-3].minor,
            k1 = self.yystack[-2].minor,
            dr = self.yystack[-1].minor,
            )
    def action_021(self):
        # command ::= K_FORCE expression T_EQDEF literal
        return self.delegate.p_command_force(
            k = self.yystack[-4].minor,
            e = self.yystack[-3].minor,
            eq = self.yystack[-2].minor,
            v = self.yystack[-1].minor,
            )
    def action_022(self):
        # command ::= K_SAMPLE expression
        return self.delegate.p_command_sample(
            k = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_023(self):
        # command ::= K_CONTINUOUS_INFERENCE_STATUS
        return self.delegate.p_command_continuous_inference_status(
            k = self.yystack[-1].minor,
            )
    def action_024(self):
        # command ::= K_START K_CONTINUOUS K_INFERENCE
        return self.delegate.p_command_start_continuous_inference(
            k0 = self.yystack[-3].minor,
            k1 = self.yystack[-2].minor,
            k2 = self.yystack[-1].minor,
            )
    def action_025(self):
        # command ::= K_STOP_CONTINUOUS_INFERENCE
        return self.delegate.p_command_stop_continuous_inference(
            k = self.yystack[-1].minor,
            )
    def action_026(self):
        # command ::= K_GET K_CURRENT K_EXCEPTION
        return self.delegate.p_command_get_current_exception(
            k0 = self.yystack[-3].minor,
            k1 = self.yystack[-2].minor,
            k2 = self.yystack[-1].minor,
            )
    def action_027(self):
        # command ::= K_GET K_STATE
        return self.delegate.p_command_get_state(
            k0 = self.yystack[-2].minor,
            k1 = self.yystack[-1].minor,
            )
    def action_028(self):
        # command ::= K_GET K_GLOBAL K_LOGSCORE
        return self.delegate.p_command_get_global_logscore(
            k0 = self.yystack[-3].minor,
            k1 = self.yystack[-2].minor,
            k2 = self.yystack[-1].minor,
            )
    def action_029(self):
        # command ::= K_LOAD L_STRING
        return self.delegate.p_command_load(
            k = self.yystack[-2].minor,
            pathname = self.yystack[-1].minor,
            )
    def action_030(self):
        # directive_ref ::= L_INTEGER
        return self.delegate.p_directive_ref_numbered(
            number = self.yystack[-1].minor,
            )
    def action_031(self):
        # directive_ref ::= L_NAME
        return self.delegate.p_directive_ref_labelled(
            label = self.yystack[-1].minor,
            )
    def action_032(self):
        # body ::= let T_SEMI expression
        return self.delegate.p_body_let(
            l = self.yystack[-3].minor,
            semi = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_033(self):
        # body ::= expression
        return self.delegate.p_body_exp(
            e = self.yystack[-1].minor,
            )
    def action_034(self):
        # let ::= let1
        return self.delegate.p_let_one(
            l = self.yystack[-1].minor,
            )
    def action_035(self):
        # let ::= let T_SEMI let1
        return self.delegate.p_let_many(
            ls = self.yystack[-3].minor,
            semi = self.yystack[-2].minor,
            l = self.yystack[-1].minor,
            )
    def action_036(self):
        # let1 ::= L_NAME T_EQDEF expression
        return self.delegate.p_let1_l(
            n = self.yystack[-3].minor,
            eq = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_037(self):
        # expression ::= do_bind
        return self.delegate.p_expression_top(
            e = self.yystack[-1].minor,
            )
    def action_038(self):
        # do_bind ::= L_NAME T_LARR expression
        return self.delegate.p_do_bind_bind(
            n = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            e = self.yystack[-1].minor,
            )
    def action_039(self):
        # do_bind ::= boolean_and
        return self.delegate.p_do_bind_none(
            e = self.yystack[-1].minor,
            )
    def action_040(self):
        # boolean_and ::= boolean_and K_AND|T_AND boolean_or
        return self.delegate.p_boolean_and_and(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_041(self):
        # boolean_and ::= boolean_or
        return self.delegate.p_boolean_and_none(
            e = self.yystack[-1].minor,
            )
    def action_042(self):
        # boolean_or ::= boolean_or K_OR|T_OR equality
        return self.delegate.p_boolean_or_or(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_043(self):
        # boolean_or ::= equality
        return self.delegate.p_boolean_or_none(
            e = self.yystack[-1].minor,
            )
    def action_044(self):
        # equality ::= equality K_EQ|T_EQ comparison
        return self.delegate.p_equality_eq(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_045(self):
        # equality ::= equality K_NEQ|T_NEQ comparison
        return self.delegate.p_equality_neq(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_046(self):
        # equality ::= comparison
        return self.delegate.p_equality_none(
            e = self.yystack[-1].minor,
            )
    def action_047(self):
        # comparison ::= comparison K_LT|T_LT additive
        return self.delegate.p_comparison_lt(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_048(self):
        # comparison ::= comparison K_LE|T_LE additive
        return self.delegate.p_comparison_le(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_049(self):
        # comparison ::= comparison K_GE|T_GE additive
        return self.delegate.p_comparison_ge(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_050(self):
        # comparison ::= comparison K_GT|T_GT additive
        return self.delegate.p_comparison_gt(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_051(self):
        # comparison ::= additive
        return self.delegate.p_comparison_none(
            e = self.yystack[-1].minor,
            )
    def action_052(self):
        # additive ::= additive K_ADD|T_ADD multiplicative
        return self.delegate.p_additive_add(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_053(self):
        # additive ::= additive K_SUB|T_SUB multiplicative
        return self.delegate.p_additive_sub(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_054(self):
        # additive ::= multiplicative
        return self.delegate.p_additive_none(
            e = self.yystack[-1].minor,
            )
    def action_055(self):
        # multiplicative ::= multiplicative K_MUL|T_MUL exponential
        return self.delegate.p_multiplicative_mul(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_056(self):
        # multiplicative ::= multiplicative K_DIV|T_DIV exponential
        return self.delegate.p_multiplicative_div(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_057(self):
        # multiplicative ::= exponential
        return self.delegate.p_multiplicative_none(
            e = self.yystack[-1].minor,
            )
    def action_058(self):
        # exponential ::= applicative K_POW|T_POW exponential
        return self.delegate.p_exponential_pow(
            l = self.yystack[-3].minor,
            op = self.yystack[-2].minor,
            r = self.yystack[-1].minor,
            )
    def action_059(self):
        # exponential ::= applicative
        return self.delegate.p_exponential_none(
            e = self.yystack[-1].minor,
            )
    def action_060(self):
        # applicative ::= applicative T_LROUND arglist T_RROUND
        return self.delegate.p_applicative_app(
            fn = self.yystack[-4].minor,
            o = self.yystack[-3].minor,
            args = self.yystack[-2].minor,
            c = self.yystack[-1].minor,
            )
    def action_061(self):
        # applicative ::= primary
        return self.delegate.p_applicative_none(
            e = self.yystack[-1].minor,
            )
    def action_062(self):
        # arglist ::=
        return self.delegate.p_arglist_none(
            )
    def action_063(self):
        # arglist ::= args
        return self.delegate.p_arglist_some(
            args = self.yystack[-1].minor,
            )
    def action_064(self):
        # args ::= expression
        return self.delegate.p_args_one(
            arg = self.yystack[-1].minor,
            )
    def action_065(self):
        # args ::= args T_COMMA expression
        return self.delegate.p_args_many(
            args = self.yystack[-3].minor,
            arg = self.yystack[-1].minor,
            )
    def action_066(self):
        # primary ::= T_LROUND body T_RROUND
        return self.delegate.p_primary_paren(
            o = self.yystack[-3].minor,
            e = self.yystack[-2].minor,
            c = self.yystack[-1].minor,
            )
    def action_067(self):
        # primary ::= T_LCURLY let T_SEMI expression T_RCURLY
        return self.delegate.p_primary_brace(
            o = self.yystack[-5].minor,
            l = self.yystack[-4].minor,
            semi = self.yystack[-3].minor,
            e = self.yystack[-2].minor,
            c = self.yystack[-1].minor,
            )
    def action_068(self):
        # primary ::= K_PROC T_LROUND paramlist T_RROUND T_LCURLY body T_RCURLY
        return self.delegate.p_primary_proc(
            k = self.yystack[-7].minor,
            po = self.yystack[-6].minor,
            params = self.yystack[-5].minor,
            pc = self.yystack[-4].minor,
            bo = self.yystack[-3].minor,
            body = self.yystack[-2].minor,
            bc = self.yystack[-1].minor,
            )
    def action_069(self):
        # primary ::= K_IF T_LROUND body T_RROUND T_LCURLY body T_RCURLY K_ELSE T_LCURLY body T_RCURLY
        return self.delegate.p_primary_if(
            k = self.yystack[-11].minor,
            po = self.yystack[-10].minor,
            p = self.yystack[-9].minor,
            pc = self.yystack[-8].minor,
            co = self.yystack[-7].minor,
            c = self.yystack[-6].minor,
            cc = self.yystack[-5].minor,
            ke = self.yystack[-4].minor,
            ao = self.yystack[-3].minor,
            a = self.yystack[-2].minor,
            ac = self.yystack[-1].minor,
            )
    def action_070(self):
        # primary ::= literal
        return self.delegate.p_primary_literal(
            l = self.yystack[-1].minor,
            )
    def action_071(self):
        # primary ::= L_NAME
        return self.delegate.p_primary_symbol(
            s = self.yystack[-1].minor,
            )
    def action_072(self):
        # paramlist ::=
        return self.delegate.p_paramlist_none(
            )
    def action_073(self):
        # paramlist ::= params
        return self.delegate.p_paramlist_some(
            params = self.yystack[-1].minor,
            )
    def action_074(self):
        # params ::= L_NAME
        return self.delegate.p_params_one(
            param = self.yystack[-1].minor,
            )
    def action_075(self):
        # params ::= params T_COMMA L_NAME
        return self.delegate.p_params_many(
            params = self.yystack[-3].minor,
            c = self.yystack[-2].minor,
            param = self.yystack[-1].minor,
            )
    def action_076(self):
        # literal ::= T_TRUE
        return self.delegate.p_literal_true(
            t = self.yystack[-1].minor,
            )
    def action_077(self):
        # literal ::= T_FALSE
        return self.delegate.p_literal_false(
            f = self.yystack[-1].minor,
            )
    def action_078(self):
        # literal ::= L_INTEGER
        return self.delegate.p_literal_integer(
            v = self.yystack[-1].minor,
            )
    def action_079(self):
        # literal ::= L_REAL
        return self.delegate.p_literal_real(
            v = self.yystack[-1].minor,
            )
    def action_080(self):
        # literal ::= L_STRING
        return self.delegate.p_literal_string(
            v = self.yystack[-1].minor,
            )
    def action_081(self):
        # literal ::= L_TAG json T_GT
        return self.delegate.p_literal_json(
            t = self.yystack[-3].minor,
            v = self.yystack[-2].minor,
            c = self.yystack[-1].minor,
            )
    def action_082(self):
        # literal ::= L_TAG error T_GT
        return self.delegate.p_literal_json_error(
            t = self.yystack[-3].minor,
            c = self.yystack[-1].minor,
            )
    def action_083(self):
        # json ::= L_STRING
        return self.delegate.p_json_string(
            v = self.yystack[-1].minor,
            )
    def action_084(self):
        # json ::= L_INTEGER
        return self.delegate.p_json_integer(
            v = self.yystack[-1].minor,
            )
    def action_085(self):
        # json ::= REAL
        return self.delegate.p_json_real(
            v = self.yystack[-1].minor,
            )
    def action_086(self):
        # json ::= json_list
        return self.delegate.p_json_list(
            l = self.yystack[-1].minor,
            )
    def action_087(self):
        # json ::= json_dict
        return self.delegate.p_json_dict(
            d = self.yystack[-1].minor,
            )
    def action_088(self):
        # json_list ::= T_LSQUARE T_RSQUARE
        return self.delegate.p_json_list_empty(
            )
    def action_089(self):
        # json_list ::= T_LSQUARE json_list_terms T_RSQUARE
        return self.delegate.p_json_list_nonempty(
            ts = self.yystack[-2].minor,
            )
    def action_090(self):
        # json_list ::= T_LSQUARE json_list_terms error T_RSQUARE
        return self.delegate.p_json_list_error1(
            ts = self.yystack[-3].minor,
            )
    def action_091(self):
        # json_list ::= T_LSQUARE error T_RSQUARE
        return self.delegate.p_json_list_error(
            )
    def action_092(self):
        # json_list_terms ::= json
        return self.delegate.p_json_list_terms_one(
            t = self.yystack[-1].minor,
            )
    def action_093(self):
        # json_list_terms ::= json_list_terms T_COMMA json
        return self.delegate.p_json_list_terms_many(
            ts = self.yystack[-3].minor,
            t = self.yystack[-1].minor,
            )
    def action_094(self):
        # json_list_terms ::= error T_COMMA json
        return self.delegate.p_json_list_terms_error(
            t = self.yystack[-1].minor,
            )
    def action_095(self):
        # json_dict ::= T_LCURLY T_RCURLY
        return self.delegate.p_json_dict_empty(
            )
    def action_096(self):
        # json_dict ::= T_LCURLY json_dict_entries T_RCURLY
        return self.delegate.p_json_dict_nonempty(
            es = self.yystack[-2].minor,
            )
    def action_097(self):
        # json_dict ::= T_LCURLY json_dict_entries error T_RCURLY
        return self.delegate.p_json_dict_error(
            es = self.yystack[-3].minor,
            )
    def action_098(self):
        # json_dict_entries ::= json_dict_entry
        return self.delegate.p_json_dict_entries_one(
            e = self.yystack[-1].minor,
            )
    def action_099(self):
        # json_dict_entries ::= json_dict_entries T_COMMA json_dict_entry
        return self.delegate.p_json_dict_entries_many(
            es = self.yystack[-3].minor,
            e = self.yystack[-1].minor,
            )
    def action_100(self):
        # json_dict_entries ::= error T_COMMA json_dict_entry
        return self.delegate.p_json_dict_entries_error(
            e = self.yystack[-1].minor,
            )
    def action_101(self):
        # json_dict_entry ::= T_STRING T_COLON json
        return self.delegate.p_json_dict_entry_e(
            key = self.yystack[-3].minor,
            value = self.yystack[-1].minor,
            )
    def action_102(self):
        # json_dict_entry ::= error T_COLON json
        return self.delegate.p_json_dict_entry_error(
            value = self.yystack[-1].minor,
            )
    yy_action_method = [
        action_000,
        action_001,
        action_002,
        action_003,
        action_004,
        action_005,
        action_006,
        action_007,
        action_008,
        action_009,
        action_010,
        action_011,
        action_012,
        action_013,
        action_014,
        action_015,
        action_016,
        action_017,
        action_018,
        action_019,
        action_020,
        action_021,
        action_022,
        action_023,
        action_024,
        action_025,
        action_026,
        action_027,
        action_028,
        action_029,
        action_030,
        action_031,
        action_032,
        action_033,
        action_034,
        action_035,
        action_036,
        action_037,
        action_038,
        action_039,
        action_040,
        action_041,
        action_042,
        action_043,
        action_044,
        action_045,
        action_046,
        action_047,
        action_048,
        action_049,
        action_050,
        action_051,
        action_052,
        action_053,
        action_054,
        action_055,
        action_056,
        action_057,
        action_058,
        action_059,
        action_060,
        action_061,
        action_062,
        action_063,
        action_064,
        action_065,
        action_066,
        action_067,
        action_068,
        action_069,
        action_070,
        action_071,
        action_072,
        action_073,
        action_074,
        action_075,
        action_076,
        action_077,
        action_078,
        action_079,
        action_080,
        action_081,
        action_082,
        action_083,
        action_084,
        action_085,
        action_086,
        action_087,
        action_088,
        action_089,
        action_090,
        action_091,
        action_092,
        action_093,
        action_094,
        action_095,
        action_096,
        action_097,
        action_098,
        action_099,
        action_100,
        action_101,
        action_102,
    ]


    def yy_reduce(self,     # The parser
                  yyruleno  # Number of the rule by which to reduce
                  ):
        '''Perform a reduce action and the shift that must immediately
        follow the reduce.'''
        
        if (self.yyTraceFILE and
            yyruleno >= 0 and yyruleno < len(self.yyRuleName)
            ):
            self.yyTraceFILE.write("%sReduce [%s].\n" % (
                self.yyTracePrompt, self.yyRuleName[yyruleno]))

        # get the action
        action = self.yy_action_method[yyruleno]

        # 'yygotominor' is the LHS of the rule reduced
        yygotominor = action(self)

        yygoto = self.yyRuleInfo[yyruleno].lhs   # The next state
        yysize = self.yyRuleInfo[yyruleno].nrhs  # Amount to pop the stack
        if yysize > 0:
            del self.yystack[-yysize:]

        # The next action
        yyact = self.yy_find_reduce_action(self.yystack[-1].stateno, yygoto)

        if yyact < self.YYNSTATE:
            self.yy_shift(yyact, yygoto, yygotominor)
        else:
            assert yyact == self.YYNSTATE + self.YYNRULE + 1
            self.yy_accept()

        return


    def yy_parse_failed(self):
        '''This method executes when the parse fails.'''

        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sFail!\n" % self.yyTracePrompt)

        while self.yystack:
            self.yy_pop_parser_stack()

        self.delegate.parse_failed()

        return


    def yy_syntax_error(self, token):
        '''This method executes when a syntax error occurs.'''
        self.delegate.syntax_error(token)
        return


    def yy_accept(self):
        '''This method executes when the parser accepts.'''

        if self.yyTraceFILE:
            self.yyTraceFILE.write("%sAccept!\n" % self.yyTracePrompt)

        while self.yystack:
            self.yy_pop_parser_stack()

        self.delegate.accept()

        return


    def parse(self, tokens):
        for token in tokens:
            self.feed(token)
        self.feed((0, None))
        return


    def feed(self, token):
        '''The main parser routine.'''

        yymajor = token[0]  # The major token code number
        yyminor = token[1]  # The value for the token

        yyerrorhit = False  # True if yymajor has invoked an error

        # (re)initialize the parser, if necessary
        if not self.yystack:
            self.yyerrcnt = -1
            yytos = self.yyStackEntry(
                stateno = 0,
                major = 0,
                minor = None
                )
            self.yystack.append(yytos)

        yyendofinput = (yymajor == 0) # True if we are at the end of input
        
        if self.yyTraceFILE:
            self.yyTraceFILE.write(
                "%sInput %s\n" %
                (self.yyTracePrompt, self.yyTokenName[yymajor]))


        cond = True
        while cond:

            # The parser action.
            yyact = self.yy_find_shift_action(yymajor)

            YYNOCODE = self.YYNOCODE
            YYNSTATE = self.YYNSTATE
            YYNRULE  = self.YYNRULE

            if yyact < YYNSTATE:
                assert not yyendofinput, "Impossible to shift the $ token"
                self.yy_shift(yyact, yymajor, yyminor)
                self.yyerrcnt -= 1
                yymajor = YYNOCODE
            elif yyact < YYNSTATE + YYNRULE:
                self.yy_reduce(yyact - YYNSTATE)
            else:
                assert yyact == self.YY_ERROR_ACTION
                if self.yyTraceFILE:
                    self.yyTraceFILE.write(
                        "%sSyntax Error!\n" % self.yyTracePrompt)

                YYERRORSYMBOL = self.YYERRORSYMBOL
                if YYERRORSYMBOL is not None:
                    # A syntax error has occurred.
                    # The response to an error depends upon whether or not the
                    # grammar defines an error token "ERROR".  
                    #
                    # This is what we do if the grammar does define ERROR:
                    #
                    #  * Call the %syntax_error function.
                    #
                    #  * Begin popping the stack until we enter a state where
                    #    it is legal to shift the error symbol, then shift
                    #    the error symbol.
                    #
                    #  * Set the error count to three.
                    #
                    #  * Begin accepting and shifting new tokens.  No new error
                    #    processing will occur until three tokens have been
                    #    shifted successfully.
                    #
                    if self.yyerrcnt < 0:
                        self.yy_syntax_error(token)

                    yymx = self.yystack[-1].major
                    if yymx == YYERRORSYMBOL or yyerrorhit:
                        if self.yyTraceFILE:
                            self.yyTraceFILE.write(
                                "%sDiscard input token %s\n" % (
                                    self.yyTracePrompt,
                                    self.yyTokenName[yymajor]))
                        yymajor = YYNOCODE
                    else:
                        while self.yystack and yymx != YYERRORSYMBOL:
                            yyact = self.yy_find_reduce_action(
                                self.yystack[-1].stateno,
                                YYERRORSYMBOL
                                )
                            if yyact < YYNSTATE:
                                break
                            self.yy_pop_parser_stack()

                        if not self.yystack or yymajor == 0:
                            self.yy_parse_failed()
                            yymajor = YYNOCODE
                        elif yymx != YYERRORSYMBOL:
                            self.yy_shift(yyact, YYERRORSYMBOL, None)

                    self.yyerrcnt = 3
                    yyerrorhit = True

                else: # YYERRORSYMBOL is not defined
                    # This is what we do if the grammar does not define ERROR:
                    #
                    #  * Report an error message, and throw away the input token.
                    #
                    #  * If the input token is $, then fail the parse.
                    #
                    # As before, subsequent error messages are suppressed until
                    # three input tokens have been successfully shifted.
                    #
                    if self.yyerrcnt <= 0:
                        self.yy_syntax_error(token)

                    self.yyerrcnt = 3
                    if yyendofinput:
                        self.yy_parse_failed()

                    yymajor = YYNOCODE

            cond = yymajor != YYNOCODE and self.yystack

        return


